/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.actions;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JTabbedPane;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.ProxySettings;
import com.eviware.soapui.settings.SSLSettings;
import com.eviware.soapui.settings.WSISettings;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.SwingConfigurationDialogImpl;
import com.eviware.soapui.support.types.StringToStringMap;

/**
 * Action for managing SoapUI preferences
 * 
 * @author Ole.Matzura
 */

public class SoapUIPreferencesAction extends AbstractAction
{
	public static final String WS_I_SETTINGS = "WS-I Settings";
	public static final String WSDL_SETTINGS = "WSDL Settings";
	public static final String UI_SETTINGS = "UI Settings";
	public static final String PROXY_SETTINGS = "Proxy Settings";
	public static final String HTTP_SETTINGS = "HTTP Settings";
	public static final String SSL_SETTINGS = "SSL Settings";
	public static final String INTEGRATED_TOOLS = "Tools";
	private SwingConfigurationDialogImpl dialog;
	private Prefs uiPrefs; 
	private Prefs wsiPrefs;
	private Prefs proxyPrefs;
	private Prefs wsdlPrefs;
	private Prefs toolsPrefs;
	private Prefs httpPrefs;
	private Prefs sslPrefs;
	private JTabbedPane tabs;
	
	private static SoapUIPreferencesAction instance;
	
	public SoapUIPreferencesAction()
	{
		super( "Preferences" );
		
		putValue( Action.SHORT_DESCRIPTION, "Sets global soapUI preferences" );
		putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu alt P"));
		
		wsiPrefs = new AnnotatedSettingsPrefs( WSISettings.class, WS_I_SETTINGS );
		proxyPrefs = new AnnotatedSettingsPrefs( ProxySettings.class, PROXY_SETTINGS );
		wsdlPrefs = new AnnotatedSettingsPrefs( WsdlSettings.class, WSDL_SETTINGS );
		sslPrefs = new AnnotatedSettingsPrefs( SSLSettings.class, SSL_SETTINGS );
		uiPrefs = new UIPrefs( UI_SETTINGS );
		toolsPrefs = new ToolsPrefs( INTEGRATED_TOOLS );
		httpPrefs = new HttpPrefs( HTTP_SETTINGS);
	
		instance = this;
	}
	
	public static SoapUIPreferencesAction getInstance()
	{
		if( instance == null )
			instance = new SoapUIPreferencesAction();
		
		return instance;
	}
	
	public void actionPerformed(ActionEvent e)
	{
		show( HTTP_SETTINGS );
	}
	
	public boolean show( String initialTab )
	{
		if( dialog == null )
			buildDialog();
	
		Settings settings = SoapUI.getSettings();
		
		httpPrefs.setFormValues( settings );
		toolsPrefs.setFormValues( settings );
		proxyPrefs.setFormValues( settings );
		wsiPrefs.setFormValues( settings );
		wsdlPrefs.setFormValues( settings );
		uiPrefs.setFormValues( settings );
		sslPrefs.setFormValues( settings );

		if( initialTab != null )
		{
			int ix = tabs.indexOfTab( initialTab );
			if( ix != -1 )
				tabs.setSelectedIndex( ix );
		}
		
		if( dialog.show( new StringToStringMap() ))
		{
         httpPrefs.getFormValues( settings );
			toolsPrefs.getFormValues( settings );
         uiPrefs.getFormValues( settings );
         proxyPrefs.getFormValues( settings );
         wsiPrefs.getFormValues( settings );
         wsdlPrefs.getFormValues( settings );
         sslPrefs.getFormValues( settings );
         
         return true;
		}
		
		return false;
	}
   
   private void buildDialog()
	{
		dialog = new SwingConfigurationDialogImpl( "soapUI Preferences", HelpUrls.PREFERENCES_HELP_URL, 
				"Set global soapUI settings", UISupport.OPTIONS_ICON );
		
		tabs = new JTabbedPane();
		tabs.addTab( HTTP_SETTINGS, httpPrefs.getForm().getPanel() );
		tabs.addTab( PROXY_SETTINGS, proxyPrefs.getForm().getPanel() );
		tabs.addTab( SSL_SETTINGS, sslPrefs.getForm().getPanel() );
		tabs.addTab( UI_SETTINGS, uiPrefs.getForm().getPanel() );
		tabs.addTab( INTEGRATED_TOOLS, toolsPrefs.getForm().getPanel() );
		tabs.addTab( WSDL_SETTINGS, wsdlPrefs.getForm().getPanel());
		tabs.addTab( WS_I_SETTINGS, wsiPrefs.getForm().getPanel() );
		dialog.setContent( UISupport.createTabPanel( tabs, false ) );
		
//		dialog.setSize( new Dimension( 450, 390 ));
		// Increased size for MacOSX
		//dialog.setSize( new Dimension( 670, 450 ));
	}

}
