/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.actions;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.WorkspaceImpl;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.support.UISupport;

/**
 * Actions for importing an existing soapui-project file into the current workspace
 * 
 * @author Ole.Matzura
 */

public class ImportWsdlProjectAction extends AbstractAction
{
   private final WorkspaceImpl workspace;

	public ImportWsdlProjectAction( WorkspaceImpl workspace )
   {
      super( "Import Project" );
		this.workspace = workspace;
      
      putValue( Action.SHORT_DESCRIPTION, "Adds an existing project into this workspace" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu I") );
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	File file = UISupport.getFileDialogs().openXML(this, "Select soapui project file");
      if( file == null ) return;
      
      String fileName = file.getAbsolutePath();
      if( fileName == null ) return;
      
      ClassLoader contextClassLoader = Thread.currentThread().getContextClassLoader();
   	Thread.currentThread().setContextClassLoader( SoapUI.class.getClassLoader() );
   	
		try
		{
			WsdlProject project = (WsdlProject) workspace.importProject(fileName);
			if( project != null )
				SoapUI.selectModelItem(project);
		}
		catch (Exception ex)
		{
			UISupport.showErrorMessage( ex );
		}      
		finally
   	{
   		Thread.currentThread().setContextClassLoader( contextClassLoader );
   	}
   }
}