/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlTestSuite;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.registry.WsdlTestRequestStepFactory;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.support.UISupport;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.support.ADialogBuilder;
import com.eviware.x.form.support.AField;
import com.eviware.x.form.support.AForm;
import com.eviware.x.form.support.AField.AFieldType;

public class GenerateTestSuiteAction extends AbstractAction
{
	private final WsdlInterface iface;

	public GenerateTestSuiteAction( WsdlInterface iface )
	{
		super( "Generate TestSuite" );
		
		putValue( Action.SHORT_DESCRIPTION, "Generates TestSuite with TestCase(s) for all Operations in this Interface");
		this.iface = iface;
	}

	public void actionPerformed( ActionEvent e )
	{
		generateTestSuite( iface );
	}

	public WsdlTestSuite generateTestSuite( WsdlInterface iface )
	{
		XFormDialog dialog = ADialogBuilder.buildDialog( GenerateForm.class );
		
		WsdlProject project = ( WsdlProject ) iface.getProject();
		String[] testSuites = ModelSupport.getNames( new String[] {"<create>"}, project.getTestSuites());
		dialog.setOptions( GenerateForm.TESTSUITE, testSuites );
		
		if( dialog.show() )
		{
			String testSuiteName = dialog.getValue( GenerateForm.TESTSUITE );
			
			if( testSuiteName.equals( "<create>" ))
				testSuiteName = UISupport.prompt( "Enter name of TestSuite to create", "Generate TestSuite", iface.getName() + " TestSuite" );
			
			if( testSuiteName != null && testSuiteName.trim().length() > 0)
			{
				WsdlTestSuite testSuite = ( WsdlTestSuite ) project.getTestSuiteByName( testSuiteName );
				
				if( testSuite == null )
				{
					testSuite = ( WsdlTestSuite ) project.addNewTestSuite( testSuiteName );
				}
				
				int style = dialog.getValueIndex( GenerateForm.STYLE );
				if( style == 0 )
					generateMulipleTestCases( testSuite, iface );
				else if( style == 1 )
					generateSingleTestCase( testSuite, iface );
				
				return testSuite;
			}
		}
		
		return null;
	}
	
	private void generateSingleTestCase( WsdlTestSuite testSuite,  WsdlInterface iface  )
	{
		WsdlTestCase testCase = testSuite.addNewTestCase( iface.getName() + " TestSuite" );
		
		for( int i = 0; i < iface.getOperationCount(); i++ )
		{
			WsdlOperation operation = ( WsdlOperation ) iface.getOperationAt( i );
			testCase.addTestStep( WsdlTestRequestStepFactory.createConfig( operation, operation.getName() ) );
		}
		
		UISupport.showDesktopPanel( testCase );
	}

	private void generateMulipleTestCases( WsdlTestSuite testSuite,  WsdlInterface iface  )
	{
		for( int i = 0; i < iface.getOperationCount(); i++ )
		{
			WsdlOperation operation = ( WsdlOperation ) iface.getOperationAt( i );
			WsdlTestCase testCase = testSuite.addNewTestCase( operation.getName() + " TestCase" );
			testCase.addTestStep( WsdlTestRequestStepFactory.createConfig( operation, operation.getName() ) );
		}
		
		UISupport.showDesktopPanel( testSuite );
	}

	@AForm( name="Generate TestSuite", description = "Generates TestSuite with TestCase(s) for all Operations in this Interface")
	private class GenerateForm 
	{
		@AField(name = "TestSuite", description = "The TestSuite to create or use", type = AFieldType.ENUMERATION )
		public final static String TESTSUITE = "TestSuite";
		
		@AField(name = "Style", description = "Select the style of TestCases to create", 
					type = AFieldType.ENUMERATION, values = { "One TestCase for each Operation", "Single TestCase with one Request for each Operation"} )
		public final static String STYLE = "Style";
	}
}
