/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;

import org.apache.log4j.Logger;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.iface.Request;
import com.eviware.soapui.support.UISupport;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Edits the service endpoints for a WsdlInterface
 * 
 * @author Ole.Matzura
 */

public class InterfaceEndpointsAction extends AbstractAction
{
   private JDialog dialog;
   private JList list;
   private WsdlInterface iface;
   private DefaultListModel listModel;
   private final static Logger log = Logger.getLogger( InterfaceEndpointsAction.class );

   public InterfaceEndpointsAction( WsdlInterface iface )
   {
      super( "Service Endpoints" );
      putValue( Action.SHORT_DESCRIPTION, "Manage service endpoints available for this interface" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu E" ));
      this.iface = iface;
   }

	private void buildDialog()
	{
		dialog = new JDialog( UISupport.getMainFrame() );
      dialog.setTitle("Interface Service Endpoints" );
      
      JPanel contentPanel = new JPanel( new BorderLayout() );
      listModel = new DefaultListModel();
      list = new JList( listModel );
      list.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
      list.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
      JScrollPane scrollPane = new JScrollPane( list );
      
      scrollPane.setBorder( BorderFactory.createCompoundBorder(
      		BorderFactory.createEmptyBorder( 0, 3, 0, 3 ), BorderFactory.createLineBorder( Color.GRAY )));
      
      scrollPane.setVerticalScrollBarPolicy( ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS );
      
      contentPanel.add( scrollPane, BorderLayout.CENTER );
      contentPanel.add( createButtons(), BorderLayout.SOUTH );
      JLabel label = new JLabel( "Edit available service endpoints for this interface in list below" );
      label.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
      contentPanel.add( label, BorderLayout.NORTH );
      
      dialog.setContentPane( contentPanel );
      dialog.setSize(400, 300);
      
      dialog.setModal( true );
	}
	
   public void actionPerformed(ActionEvent e)
	{
   	if( dialog == null )
   		buildDialog();
   	
      listModel.clear();
      
      String[] endpoints = iface.getEndpoints();
      for( int c = 0; c < endpoints.length; c++ )
      {
         listModel.addElement( endpoints[c] );
      }
      
      UISupport.showDialog( dialog );
   }

   private Component createButtons()
   {
      ButtonBarBuilder builder = ButtonBarBuilder.createLeftToRightBuilder();
      builder.addFixed( new JButton( new AddAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new EditAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new DeleteAction() ));
      builder.addRelatedGap();
      builder.addFixed( new JButton( new AssignAction() ));
      builder.addGlue();
      builder.addFixed( new JButton( new OkAction() ));
      builder.addRelatedGap();
		builder.addFixed( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.ENDPOINTSEDITOR_HELP_URL )));

      builder.getPanel().setBorder( BorderFactory.createEmptyBorder(5, 5, 5, 5));
      return builder.getPanel();
   }
   
   private class AddAction extends AbstractAction
   {
      public AddAction()
      {
         super( "Add" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         Object selectedValue = list.getSelectedValue();
			String endpoint = UISupport.prompt( "Enter new endpoint address", "Add Endpoint", 
         		selectedValue == null ? "" : selectedValue.toString() );
			
         if( endpoint == null ) return;
         
         listModel.addElement( endpoint );
         iface.addEndpoint( endpoint );
      }
   }
   
   private class AssignAction extends AbstractAction
   {
      private static final String ALL_REQUESTS = "- all requests -";
		private static final String ALL_REQUESTS_WITH_NO_ENDPOINT = "- all requests with no endpoint -";

		public AssignAction()
      {
         super( "Assign" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
      	int selectedIndex = list.getSelectedIndex();
         if( selectedIndex == -1 ) 
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         
         String selectedEndpoint = (String) listModel.getElementAt( selectedIndex );
         
         List<String> list = new ArrayList<String>( Arrays.asList( iface.getEndpoints()) );
         list.add( 0, ALL_REQUESTS );
         list.add( 0, ALL_REQUESTS_WITH_NO_ENDPOINT );
         
         Object endpoint = UISupport.prompt(	"Assign selected endpoint to..", "Assign Endpoint", 
         		 list.toArray(), ALL_REQUESTS_WITH_NO_ENDPOINT );
         
         if( endpoint == null )
         	return;
         
         int changeCount = 0;
         
         for( int c = 0; c < iface.getOperationCount(); c++ )
         {
         	Operation operation = iface.getOperationAt( c );
         	for( int i = 0; i < operation.getRequestCount(); i++ )
         	{
         		Request request = operation.getRequestAt( i );
         		String ep = request.getEndpoint();

               if( endpoint.equals( ALL_REQUESTS ) || 
               	 ( endpoint.equals(  ALL_REQUESTS_WITH_NO_ENDPOINT ) && ep == null ) ||
               	 ( ep.equals( endpoint )))
               {
               	request.setEndpoint( selectedEndpoint );
               	changeCount++;
               }
         	}
         }
         
         log.info( "Assigned endpoint [" + selectedEndpoint + "] to " + changeCount + " requests" );
      }
   }
   
   private class EditAction extends AbstractAction
   {
      public EditAction()
      {
         super( "Edit" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         int selectedIndex = list.getSelectedIndex();
         if( selectedIndex == -1 ) 
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         	
         String oldEndpoint = (String) listModel.getElementAt( selectedIndex );
         String newEndpoint = UISupport.prompt( "Edit endpoint address", "Edit Endpoint", oldEndpoint );
         if( newEndpoint == null ) return;
         
         listModel.setElementAt( newEndpoint, selectedIndex );
         iface.changeEndpoint( oldEndpoint, newEndpoint );
      }
   }
   
   private class DeleteAction extends AbstractAction
   {
      public DeleteAction()
      {
         super( "Delete" );
      }
      
      public void actionPerformed(ActionEvent e )
      {   
         int index = list.getSelectedIndex();
         if( index == -1)
         {
         	Toolkit.getDefaultToolkit().beep();
         	return;
         }
         
         if( UISupport.confirm( "Delete selected endpoint?", "Delete Endpoint" ))
         {
	         String oldEndpoint = (String) listModel.getElementAt( index );
	         listModel.removeElementAt( index );
	         iface.removeEndpoint( oldEndpoint );
         }
      }
   }

   private class OkAction extends AbstractAction
   {
      public OkAction()
      {
         super( "OK" );
      }
      
      public void actionPerformed(ActionEvent e)
      {
         dialog.setVisible( false );
      }
   }
}
