/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.support.UISupport;

/**
 * Removes a WsdlInterface from a WsdlProject
 * 
 * @author Ole.Matzura
 */

public class RemoveInterfaceAction extends AbstractAction
{
   private final WsdlInterface iface;

	public RemoveInterfaceAction( WsdlInterface iface )
   {
      super( "Remove" );
		this.iface = iface;
      putValue( Action.SHORT_DESCRIPTION, "Removes this interface from the project" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "DELETE" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	if( hasDependingTests( iface ) )
   	{
   		UISupport.showErrorMessage( "Cannot remove Interface due to running depending tests" );
   		return;
   	}
   	
      if( UISupport.confirm( "Remove interface [" + iface.getName() + "] from project [" + 
            iface.getProject().getName() + "]?", "Remove Interface" ))
      {
      	WsdlProject project = (WsdlProject) iface.getProject();
         project.removeInterface( iface);
      }
   }

	static boolean hasDependingTests( WsdlInterface iface )
	{
		if( SoapUI.getTestMonitor() == null )
			return false;
		
		for( int c = 0; c < iface.getProject().getTestSuiteCount(); c++ )
		{
			TestSuite testSuite = iface.getProject().getTestSuiteAt( c );
			for( int i = 0; i < testSuite.getTestCaseCount(); i++ )
			{
				TestCase testCase = testSuite.getTestCaseAt( i );
				if( !SoapUI.getTestMonitor().hasRunningTest( testCase ))
					continue;
				
				for( int j = 0; j < testCase.getTestStepCount(); j++ )
				{
					WsdlTestStep testStep = (WsdlTestStep) testCase.getTestStepAt( j );
					if( testStep.dependsOn( iface ))
					{
						return true;
					}
				}
			}
		}
		
		return false;
	}
}
