/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.iface;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlInterface;
import com.eviware.soapui.support.UISupport;

/**
 * Updates the definition of a WsdlInterface.
 * 
 * @author Ole.Matzura
 */

public class UpdateInterfaceAction extends AbstractAction
{
	private final WsdlInterface iface;
   
   public UpdateInterfaceAction( WsdlInterface iface )
   {
      super("Update Definition");
		this.iface = iface;
      putValue( Action.SHORT_DESCRIPTION, "Reloads the definition for this interface and its operations");
      putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/updateDefinition.gif" ));
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "F5" ));
   }

   public void actionPerformed(ActionEvent e)
	{
   	if( RemoveInterfaceAction.hasDependingTests( iface ))
   	{
   		UISupport.showErrorMessage( "Cannot update Interface due to running depending tests" );
   		return;
   	}
   	
      String url = UISupport.prompt( "Refresh from specified url:", "Update Definition", iface.getDefinition());
      if (url == null)
         return;

      boolean createRequests = UISupport.confirm( "Create default requests for new methods?", "Update Definition" );
      
      try
		{
      	if( iface.updateDefinition( url, createRequests ))
      		UISupport.showInfoMessage(	"Update of interface successfull!",  "Update Definition" );
      	else
      		UISupport.showInfoMessage(	"Update of interface failed",  "Update Definition" );
		}
		catch (Exception e1)
		{
			UISupport.showInfoMessage(	"Failed to update interface: [" + e1 + "]", "Update Definition" );
			e1.printStackTrace();
		}
   } 
}
