/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.loadtest;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.loadtest.WsdlLoadTest;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.model.support.LoadTestRunListenerAdapter;
import com.eviware.soapui.model.testsuite.LoadTestRunContext;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.support.UISupport;

/**
 * Removes a WsdlLoadTest from its WsdlTestSuite
 * 
 * @author Ole.Matzura
 */

public class RemoveLoadTestAction extends AbstractAction
{
	private final WsdlLoadTest loadTest;
	private final InternalLoadTestRunListener internalLoadTestRunListener = new InternalLoadTestRunListener();
	
	public RemoveLoadTestAction( WsdlLoadTest loadTest )
   {
      super( "Remove" );
		this.loadTest = loadTest;
      putValue( Action.SHORT_DESCRIPTION, "Removes this Test Schedule from the test-case" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "DELETE" ));
      
		loadTest.addLoadTestRunListener( internalLoadTestRunListener);
   }
   
   public void actionPerformed(ActionEvent e)
	{
      if( UISupport.confirm( "Remove LoadTest [" + loadTest.getName() + "] from test-casee", "Remove LoadTest" ))
      {
      	((WsdlTestCase)loadTest.getTestCase()).removeLoadTest( loadTest );
      }
   }
   
   private final class InternalLoadTestRunListener extends LoadTestRunListenerAdapter
	{
		public void beforeLoadTest(LoadTestRunner loadTestRunner, LoadTestRunContext context)
		{
			setEnabled( false );
		}

		public void afterLoadTest(LoadTestRunner loadTestRunner, LoadTestRunContext context)
		{
			setEnabled( true );
		}
	}
}