/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.mockservice;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.support.UISupport;

/**
 * Removes a WsdlMockService from its WsdlMockService
 * 
 * @author Ole.Matzura
 */

public class RemoveMockServiceAction extends AbstractAction
{
   private final WsdlMockService mockService;

	public RemoveMockServiceAction( WsdlMockService mockService )
   {
      super( "Remove" );
		this.mockService = mockService;
      putValue( Action.SHORT_DESCRIPTION, "Removes this MockService from the MockService" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "DELETE" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
   	if( SoapUI.getMockEngine().hasRunningMock( mockService ))
   	{
   		UISupport.showErrorMessage( "Cannot remove MockService while mocks are running" );
   		return;
   	}
   	
      if( UISupport.confirm( "Remove MockService [" + mockService.getName() + "] from MockService", "Remove MockService" ))
      {
      	((WsdlProject)mockService.getProject()).removeMockService( mockService );
      }
   }

}
