/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.operation;

import java.awt.event.ActionEvent;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.actions.iface.AbstractSwingAction;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockOperation;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockResponse;
import com.eviware.soapui.impl.wsdl.mock.WsdlMockService;
import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.support.ModelSupport;
import com.eviware.soapui.support.UISupport;

public class AddToMockServiceAction extends AbstractSwingAction<WsdlOperation>
{
	private static final String CREATE_MOCKSUITE_OPTION = "Create new..";
	private final WsdlRequest request;

	public AddToMockServiceAction( WsdlOperation operation, WsdlRequest request )
	{
		super( "Add to MockService", request == null ? "Add this operation to a MockService" :
			"Adds the current response to a MockService", "/addToMockService.gif", operation );

		this.request = request;
	}

	public AddToMockServiceAction( WsdlOperation operation )
	{
		this( operation, null );
	}
	
	public AddToMockServiceAction( WsdlRequest request )
	{
		this( ( WsdlOperation ) request.getOperation(), request );
	}
	
	@Override
	public void actionPerformed( ActionEvent arg0, WsdlOperation operation )
	{
		String title = getName();

		if( request != null && request.getResponse() == null )
		{
			if( !UISupport.confirm( "Request is missing response, create default mock response instead?", title ))
			{
				return;
			}
		}

		WsdlMockService mockService = null;
		WsdlProject project = ( WsdlProject ) operation.getInterface().getProject();
		
		while( mockService == null )
		{
			if( project.getMockServiceCount() > 0 )
			{
				String[] mockServices = ModelSupport.getNames( project.getMockServices(), new String[] {CREATE_MOCKSUITE_OPTION});
	
				// prompt
				String option = ( String ) UISupport.prompt( "Select MockService for MockOperation", title, mockServices );
				if( option == null )
					return;
				
				mockService = ( WsdlMockService ) project.getMockServiceByName( option );
			}
	
			// create new mocksuite?
			if( mockService == null )
			{
				String mockServiceName = UISupport.prompt( "Enter name of new MockService", title, "MockService " + (project.getMockServiceCount()+1) );
				if( mockServiceName == null || mockServiceName.trim().length() == 0 )
					return;
				
				mockService = ( WsdlMockService ) project.addNewMockService( mockServiceName );
			}
			
			if( mockService.hasMockOperation( operation ))
			{
				UISupport.showErrorMessage( "MockService [" + mockService.getName() + "] already has a MockOperation for [" +  
							operation.getName() + "], please select another MockService" );
				mockService = null;
			}
		}
		
		// add mockoperation
		WsdlMockOperation mockOperation = ( WsdlMockOperation ) mockService.addNewMockOperation( operation );
		WsdlMockResponse mockResponse = mockOperation.addNewMockResponse( "Response 1", false );
		
		// add expected response if available
		if( request != null && request.getResponse() != null )
		{
			WsdlResponse response = ( WsdlResponse ) request.getResponse();
			mockResponse.setResponseContent( response.getContentAsString() );
			
			Attachment[] attachments = response.getAttachments();
			for( Attachment attachment : attachments )
			{
				mockResponse.addAttachment( attachment );
			}
			
			if( UISupport.confirm( "Add HTTP Headers to MockResponse", title ))
				mockResponse.setResponseHeaders( response.getResponseHeaders() );
		}
		else
		{
			mockResponse.setResponseContent( operation.createResponse( true ));
		}
		
		if( UISupport.confirm( "Open MockResponse editor?", title ))
		{
			SoapUI.getDesktop().showDesktopPanel( mockResponse );
		}
	}
}
