/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.operation;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.settings.WsdlSettings;
import com.eviware.soapui.support.UISupport;

/**
 * Adds a new WsdlRequest to a WsdlOperation
 * 
 * @author Ole.Matzura
 */

public class NewRequestAction extends AbstractAction
{
   private final WsdlOperation operation;

	public NewRequestAction( WsdlOperation operation )
   {
      super( "New request" );
		this.operation = operation;
      putValue( Action.SHORT_DESCRIPTION, "Creates a new request for this operation" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu N" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      String name = UISupport.prompt( "Specify name of request", "New request", "Request " + (operation.getRequestCount()+1) );
      if( name == null ) return;
      
      boolean createOptional = operation.getSettings().getBoolean(WsdlSettings.XML_GENERATION_ALWAYS_INCLUDE_OPTIONAL_ELEMENTS);
      if (!createOptional)
    	  createOptional =  UISupport.confirm(	"Create optional elements in schema?", "Create Request" );
      
      WsdlRequest newRequest = operation.addNewRequest( name );
      String requestContent = operation.createRequest( createOptional );
      if(requestContent != null)
      {
         newRequest.setRequestContent( requestContent );
      }
      
      UISupport.showDesktopPanel( newRequest );
   }
}
