/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.actions.project;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.model.iface.Interface;
import com.eviware.soapui.support.UISupport;

/**
 * Adds a WsdlInterface to a WsdlProject from a URL
 * 
 * @author Ole.Matzura
 */

public class AddInterfaceActionFromURL extends AbstractAction
{
   private final WsdlProject project;

	public AddInterfaceActionFromURL( WsdlProject project ) 
   {
      super( "Add WSDL from URL" );
		this.project = project;
      putValue( Action.SHORT_DESCRIPTION, "Adds all interfaces in a specified WSDL URL to the current project" );
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "menu U" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      String url = UISupport.prompt( "Enter WSDL URL", "Add WSDL from URL", "" );
      if( url == null ) return;
      
		try
		{
			Boolean createRequests = UISupport.confirmOrCancel( "Create default requests for all operations", "Import WSDL" );
			if( createRequests == null )
				return;
			
			Interface[] ifaces = project.importWsdl(url, createRequests);
			if (ifaces.length > 0)
				SoapUI.selectModelItem(ifaces[0]);
		}
		catch (Exception ex)
		{
			UISupport.showErrorMessage( ex.getMessage() + ":" + ex.getCause() );
		}      
   }
}
