 /*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.loadtest.log;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Date;

import javax.swing.ImageIcon;

import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestStepResult;
import com.eviware.soapui.support.action.ActionList;

/**
 * An error entry in the LoadTest Log
 * 
 * @author Ole.Matzura
 */

public class LoadTestLogErrorEntry implements LoadTestLogEntry
{
	private final String error;
	private TestStepResult result;
	private String type;
	private ImageIcon icon;
	private long timestamp;

	public LoadTestLogErrorEntry( String type, String error, TestStepResult result, ImageIcon icon)
	{
		this.icon = icon;
		this.type = type;
		this.error = error;
		this.result = result;
		
		timestamp = result == null ? System.currentTimeMillis() : result.getTimeStamp();
	}

	public LoadTestLogErrorEntry(String type, String message, ImageIcon icon)
	{
	   this.type = type;
		this.error = message;
		this.icon = icon;
		
		timestamp = System.currentTimeMillis();
	}

	public String getMessage()
	{
		return error;
	}

	public TestStepResult getTestStepResult()
	{
		return result;
	}
	
	public long getTimeStamp()
	{
		return timestamp;
	}

	public TestStep getTargetStep()
	{
		return result == null ? null : result.getTestStep();
	}

	public ImageIcon getIcon()
	{
		return icon;
	}

	public String getType()
	{
		return type;
	}

	public boolean isError()
	{
		return true;
	}

	public ActionList getActions()
	{
		return result == null ? null : result.getActions();
	}
	
	public void exportToFile(String fileName) throws IOException
	{
		PrintWriter writer = new PrintWriter(fileName );

		writer.write( new Date( timestamp ).toString() );
		writer.write( ":" );
		writer.write( error );
		writer.println();
		if( result != null )
		{
			writer.println( "----------------------------------------------------" );
			result.writeTo( writer );
		}
		
		writer.close();
	}
}
