/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.mock;

import java.util.Collection;
import java.util.Map;
import java.util.Set;

import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.model.mock.MockRunContext;
import com.eviware.soapui.model.mock.MockService;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.support.types.StringToObjectMap;
import com.eviware.soapui.support.types.StringToStringMap;

public class WsdlMockRunContext implements MockRunContext, Map<String,Object>, TestRunContext
{
	private StringToObjectMap properties;
	private final WsdlMockService mockService;
	private final WsdlTestRunContext context; 
	
	public WsdlMockRunContext( WsdlMockService mockService, WsdlTestRunContext context )
	{
		this.mockService = mockService;
		this.context = context;
		
		properties = context == null ? new StringToObjectMap() : context.getProperties(); 
	}

	public MockService getMockService()
	{
		return mockService;
	}

	public Object getProperty( String name )
	{
		return properties.get( name );
	}

	public boolean hasProperty( String name )
	{
		return properties.containsKey( name );
	}

	public Object removeProperty( String name )
	{
		return properties.remove( name );
	}

	public void setProperty( String name, Object value )
	{
		properties.put( name, value );
	}

	public StringToStringMap toStringToStringMap()
	{
		StringToStringMap result = new StringToStringMap();
		
		for( String key : properties.keySet() )
		{
			Object value = properties.get( key );
			if( value != null )
				result.put( key, value.toString() );
		}
		
		return result;
	}

	public void clear()
	{
		properties.clear();
	}

	public Object clone()
	{
		return properties.clone();
	}

	public boolean containsKey( Object arg0 )
	{
		return properties.containsKey( arg0 );
	}

	public boolean containsValue( Object arg0 )
	{
		return properties.containsValue( arg0 );
	}

	public Set<Entry<String, Object>> entrySet()
	{
		return properties.entrySet();
	}

	public boolean equals( Object arg0 )
	{
		return properties.equals( arg0 );
	}

	public Object get( Object arg0 )
	{
		return properties.get( arg0 );
	}

	public int hashCode()
	{
		return properties.hashCode();
	}

	public boolean isEmpty()
	{
		return properties.isEmpty();
	}

	public Set<String> keySet()
	{
		return properties.keySet();
	}

	public Object put( String arg0, Object arg1 )
	{
		return properties.put( arg0, arg1 );
	}

	public void putAll( Map<? extends String, ? extends Object> arg0 )
	{
		properties.putAll( arg0 );
	}

	public Object remove( Object arg0 )
	{
		return properties.remove( arg0 );
	}

	public int size()
	{
		return properties.size();
	}

	public String toString()
	{
		return properties.toString();
	}

	public Collection<Object> values()
	{
		return properties.values();
	}

	public TestStep getCurrentStep()
	{
		return context == null ? null : context.getCurrentStep();
	}

	public int getCurrentStepIndex()
	{
		return context == null ? -1 : context.getCurrentStepIndex();
	}

	public Object getProperty( String testStep, String propertyName )
	{
		return context == null ? null : context.getProperty( testStep, propertyName );
	}

	public TestRunner getTestRunner()
	{
		return context == null ? null : context.getTestRunner();
	}

	public TestCase getTestCase()
	{
		// TODO Auto-generated method stub
		return null;
	}
}