/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.mock;

import groovy.lang.GroovyShell;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.swing.ImageIcon;

import com.eviware.soapui.config.MockOperationConfig;
import com.eviware.soapui.config.MockServiceConfig;
import com.eviware.soapui.impl.actions.ShowDesktopPanelAction;
import com.eviware.soapui.impl.wsdl.AbstractWsdlModelItem;
import com.eviware.soapui.impl.wsdl.WsdlOperation;
import com.eviware.soapui.impl.wsdl.WsdlProject;
import com.eviware.soapui.impl.wsdl.actions.mockservice.AddNewMockOperationAction;
import com.eviware.soapui.impl.wsdl.actions.mockservice.CloneMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.mockservice.RemoveMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.mockservice.RenameMockServiceAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.support.ModelItemIconAnimator;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestRunContext;
import com.eviware.soapui.model.iface.Operation;
import com.eviware.soapui.model.mock.MockResult;
import com.eviware.soapui.model.mock.MockRunListener;
import com.eviware.soapui.model.mock.MockRunner;
import com.eviware.soapui.model.mock.MockService;
import com.eviware.soapui.model.mock.MockServiceListener;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.support.ScriptingSupport;
import com.eviware.soapui.support.action.ActionSupport;

public class WsdlMockService extends AbstractWsdlModelItem<MockServiceConfig> implements MockService
{
	private static final String REQUIRE_SOAP_VERSION = WsdlMockService.class.getName() + "@require-soap-version";
	private List<WsdlMockOperation> mockOperations = new ArrayList<WsdlMockOperation>();
   private Set<MockRunListener> mockRunListeners = new HashSet<MockRunListener>();
   private Set<MockServiceListener> mockServiceListeners = new HashSet<MockServiceListener>();
   private MockServiceIconAnimator iconAnimator;
	private WsdlMockRunner mockRunner;
	private GroovyShell shell;
	
	public WsdlMockService( Project project, MockServiceConfig config)
	{
		super( config, project, "/mockService.gif" );
		
		List<MockOperationConfig> testStepConfigs = config.getMockOperationList();
      for (MockOperationConfig tsc : testStepConfigs )
      {
         WsdlMockOperation testStep = new WsdlMockOperation( this, tsc );
         mockOperations.add( testStep );
      }
      
      if( !config.isSetPort() || config.getPort() < 1 )
      	config.setPort( 8080 );
      
      if( !config.isSetPath() )
      	config.setPath( "/" );
      
      addAction( new ShowDesktopPanelAction( "Open MockService Editor", 
         		"Opens the MockService Editor for this MockService", this ));
     // addAction( new MockServiceOptionsAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new AddNewMockOperationAction( this ));
      addAction( new CloneMockServiceAction( this ));
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new RenameMockServiceAction( this ) );
      addAction( new RemoveMockServiceAction( this ) );
      addAction( ActionSupport.SEPARATOR_ACTION );
      addAction( new ShowOnlineHelpAction( HelpUrls.MOCKSERVICE_HELP_URL ));
      
      iconAnimator = new MockServiceIconAnimator();
      addMockRunListener( iconAnimator );
      
      shell = ScriptingSupport.createGroovyShell( null );
	}

	public void addMockRunListener( MockRunListener listener )
	{
		mockRunListeners.add( listener );
	}

	public String getPath()
	{
		return getConfig().getPath();
	}

	public WsdlMockOperation getMockOperationAt( int index )
	{
		return mockOperations.get( index );
	}

	public WsdlMockOperation getMockOperationByName( String name )
	{
		return ( WsdlMockOperation ) getWsdlModelItemByName( mockOperations, name );
	}

	public int getMockOperationCount()
	{
		return mockOperations.size();
	}

	public WsdlProject getProject()
	{
		return ( WsdlProject ) getParent();
	}

	public int getPort()
	{
		return getConfig().getPort();
	}

	public void removeMockRunListener( MockRunListener listener )
	{
		mockRunListeners.remove( listener );
	}

	public WsdlMockRunner start( WsdlTestRunContext context ) throws Exception
	{
		String path = getPath();
		if( path == null || path.trim().length() == 0 || path.trim().charAt( 0 ) != '/' )
			throw new Exception( "Invalid path; must start with '/'");
		
		mockRunner = new WsdlMockRunner( this, context );
		return mockRunner;
	}

	public WsdlMockRunner getMockRunner()
	{
		return mockRunner;
	}

	public boolean hasMockOperation( Operation operation )
	{
		for( int c = 0; c < getMockOperationCount(); c++ )
		{
			WsdlMockOperation mockOperation = mockOperations.get( c );
			if( mockOperation.getOperation() == operation )
				return true;
		}
		
		return false;
	}

	public WsdlMockOperation addNewMockOperation( WsdlOperation operation )
	{
		if( hasMockOperation( operation ))
			return null;
		
		MockOperationConfig config = getConfig().addNewMockOperation();
		config.setName( operation.getName() );
		WsdlMockOperation mockOperation = new WsdlMockOperation( this, config, operation );
		
		mockOperations.add( mockOperation );
		fireMockOperationAdded( mockOperation );
		
		return mockOperation;
	}

	public void setPort( int port )
	{
		int oldPort = getPort();
		if( port != oldPort )
		{
			getConfig().setPort( port );
			notifyPropertyChanged( PORT_PROPERTY, oldPort, port );
		}
	}

	public void release()
	{
		super.release();
		
		for( WsdlMockOperation operation : mockOperations )
			operation.release();
	}
	
	public void setPath( String path )
	{
		String oldPath = getPath();
		if( !path.equals( oldPath ))
		{
			getConfig().setPath( path );
			notifyPropertyChanged( PATH_PROPERTY, oldPath, path );
		}
	}

	public MockRunListener [] getMockRunListeners()
	{
		return mockRunListeners.toArray( new MockRunListener[mockRunListeners.size()] );
	}

	public void removeMockOperation( WsdlMockOperation mockOperation )
	{
		int ix = mockOperations.indexOf( mockOperation );
		if( ix == -1 )
			throw new RuntimeException( "Unkonws MockOperation specified to removeMockOperation" );
		
		mockOperations.remove( ix );
		fireMockOperationRemoved( mockOperation );
		mockOperation.release();
		getConfig().removeMockOperation( ix );
	}

	public void addMockServiceListener( MockServiceListener listener )
	{
		mockServiceListeners.add(  listener  );
	}

	public void removeMockServiceListener( MockServiceListener listener )
	{
		mockServiceListeners.remove( listener );
	}
	
	protected void fireMockOperationAdded( WsdlMockOperation mockOperation )
	{
		MockServiceListener[] listeners = mockServiceListeners.toArray( new MockServiceListener[mockServiceListeners.size()] );
		for( MockServiceListener listener : listeners )
		{
			listener.mockOperationAdded( mockOperation );
		}
	}
	
	protected void fireMockOperationRemoved( WsdlMockOperation mockOperation )
	{
		MockServiceListener[] listeners = mockServiceListeners.toArray( new MockServiceListener[mockServiceListeners.size()] );
		for( MockServiceListener listener : listeners )
		{
			listener.mockOperationRemoved( mockOperation );
		}
	}
	
	protected void fireMockResponseAdded( WsdlMockResponse mockResponse )
	{
		MockServiceListener[] listeners = mockServiceListeners.toArray( new MockServiceListener[mockServiceListeners.size()] );
		for( MockServiceListener listener : listeners )
		{
			listener.mockResponseAdded( mockResponse );
		}
	}
	
	protected void fireMockResponseRemoved( WsdlMockResponse mockResponse )
	{
		MockServiceListener[] listeners = mockServiceListeners.toArray( new MockServiceListener[mockServiceListeners.size()] );
		for( MockServiceListener listener : listeners )
		{
			listener.mockResponseRemoved( mockResponse );
		}
	}

	@Override
	public ImageIcon getIcon()
	{
		return iconAnimator.getIcon();
	}

	private class MockServiceIconAnimator extends ModelItemIconAnimator implements MockRunListener
	{
		public MockServiceIconAnimator()
		{
			super( WsdlMockService.this, "/mockService.gif", 
					new String[] {"/mockService_1.gif", "/mockService_2.gif",
						"/mockService_3.gif", "/mockService_4.gif"} );
		}

		public void onMockRequest( MockRunner runner, HttpServletRequest request, HttpServletResponse response )
		{
		}

		public void onMockResult( MockResult result )
		{
		}

		public void onMockRunnerStart( MockRunner mockRunner )
		{
			start();
		}

		public void onMockRunnerStop( MockRunner mockRunner )
		{
			stop();
			WsdlMockService.this.mockRunner = null; 
		}
	}
	
	public String getLocalEndpoint()
	{
		return "http://127.0.0.1:" + getPort() + getPath();
	}

	public GroovyShell getScriptShell()
	{
		return shell;
	}

	public boolean isRequireSoapVersion()
	{
		return getSettings().getBoolean( REQUIRE_SOAP_VERSION );
	}
	
	public void setRequireSoapVersion( boolean requireSoapVersion )
	{
	   getSettings().setBoolean( REQUIRE_SOAP_VERSION, requireSoapVersion );	
	}

	public WsdlMockRunner start() throws Exception
	{
		return start( null );
	}
}
