/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.loadtest;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JSpinner;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.config.LoadTestLimitTypesConfig;
import com.eviware.soapui.impl.wsdl.actions.loadtest.LoadTestOptionsAction;
import com.eviware.soapui.impl.wsdl.actions.support.ShowOnlineHelpAction;
import com.eviware.soapui.impl.wsdl.loadtest.WsdlLoadTest;
import com.eviware.soapui.impl.wsdl.loadtest.WsdlLoadTestRunner;
import com.eviware.soapui.impl.wsdl.loadtest.data.LoadTestStatistics;
import com.eviware.soapui.impl.wsdl.loadtest.data.LoadTestStatistics.Statistic;
import com.eviware.soapui.impl.wsdl.loadtest.data.actions.ExportStatisticsAction;
import com.eviware.soapui.impl.wsdl.loadtest.log.LoadTestLog;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategy;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategyFactory;
import com.eviware.soapui.impl.wsdl.loadtest.strategy.LoadStrategyRegistry;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.support.LoadTestRunListenerAdapter;
import com.eviware.soapui.model.testsuite.LoadTestRunContext;
import com.eviware.soapui.model.testsuite.LoadTestRunListener;
import com.eviware.soapui.model.testsuite.LoadTestRunner;
import com.eviware.soapui.model.testsuite.LoadTestRunner.Status;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.components.JXToolBar;
import com.eviware.soapui.ui.desktop.DesktopPanel;
import com.eviware.soapui.ui.support.DesktopListenerAdapter;
import com.eviware.soapui.ui.support.ModelItemDesktopPanel;
import com.jgoodies.forms.builder.ButtonBarBuilder;

/**
 * Desktop panel for LoadTests
 * 
 * @author Ole.Matzura
 */

public class WsdlLoadTestDesktopPanel extends ModelItemDesktopPanel<WsdlLoadTest> implements PropertyChangeListener
{
	private JPanel contentPanel;
	private JSplitPane mainSplit;
	private JTabbedPane mainTabs;
	private JPanel graphPanel;
	private JButton runButton;
	private JButton cancelButton;
	private JButton statisticsGraphButton;
	private WsdlLoadTestRunner runner;
	private JSpinner threadsSpinner;
	private LoadTestRunListener internalLoadTestListener = new InternalLoadTestListener();
	private JComboBox strategyCombo;
	private JPanel loadStrategyConfigurationPanel;
	private JButton resetButton;
	private LoadTestLog loadTestLog;
	private JButton optionsButton;
	private JButton testTimesGraphButton;
	private Object limit;
	private JSpinner limitSpinner;
	private JComboBox limitTypeCombo;
	private SpinnerNumberModel limitSpinnerModel;
	private JProgressBar progressBar;
	private long loadTestStartTime;
	private StatisticsDesktopPanel statisticsDesktopPanel;
	private StatisticsHistoryDesktopPanel statisticsHistoryDesktopPanel;
	
	public boolean loadTestIsRunning;
	private InternalDesktopListener desktopListener;
	private JButton exportButton;
	private JLoadTestAssertionsTable assertionsTable;
	
	public WsdlLoadTestDesktopPanel(WsdlLoadTest loadTest)
	{
		super( loadTest );
		
		loadTestLog = loadTest.getLoadTestLog();
		loadTest.addPropertyChangeListener( WsdlLoadTest.THREADCOUNT_PROPERTY, this );
		loadTest.addLoadTestRunListener( internalLoadTestListener );
		
		desktopListener = new InternalDesktopListener();
	//	SoapUI.getDesktop().addDesktopListener( desktopListener );
		
		buildUI();
	}

	private void buildUI()
	{
		contentPanel = new JPanel( new BorderLayout());
		
		contentPanel.add( buildToolbar(), BorderLayout.NORTH );
		contentPanel.add( buildContent(), BorderLayout.CENTER );
		
		contentPanel.setPreferredSize( new Dimension( 600, 500 ));
	}

	private JComponent buildContent()
	{
		JTabbedPane tabbedPane = new JTabbedPane( JTabbedPane.BOTTOM );
		tabbedPane.addTab( "LoadTest Log", buildLog() );
		tabbedPane.addTab( "LoadTest Assertions", buildAssertions() );
		JPanel p = new JPanel( new BorderLayout() );
		p.add( tabbedPane, BorderLayout.CENTER );
		p.setBackground( Color.LIGHT_GRAY );
		
		JSplitPane mainSplit = UISupport.createVerticalSplit( buildStatistics(), p );
		mainSplit.setDividerLocation( 150 );
		return mainSplit;
	}

	private JComponent buildStatistics(	)
	{
		return new JStatisticsTable( getModelItem() );
	}

	private JComponent buildLog()
	{
		JLoadTestLogTable loadTestLogTable = new JLoadTestLogTable( loadTestLog );
		return loadTestLogTable;
	}

	private JComponent buildAssertions()
	{
		assertionsTable = new JLoadTestAssertionsTable( getModelItem() );
		return assertionsTable;
	}

	private JComponent buildToolbar()
	{
		WsdlLoadTest loadTest = getModelItem();
		
		JXToolBar toolbar = UISupport.createToolbar();
		
		//ButtonBarBuilder builder = new ButtonBarBuilder();
      runButton = UISupport.createToolbarButton( new RunLoadTestAction());
      cancelButton = UISupport.createToolbarButton( new CancelRunTestCaseAction(), false );
      resetButton = UISupport.createToolbarButton( new ResetAction() );
      exportButton = UISupport.createToolbarButton( new ExportStatisticsAction( loadTest.getStatisticsModel() ) );
      
      statisticsGraphButton = UISupport.createToolbarButton( new ShowStatisticsGraphAction() );
      testTimesGraphButton = UISupport.createToolbarButton( new ShowTestTimesGraphAction() );
      optionsButton = UISupport.createToolbarButton( new LoadTestOptionsAction( getModelItem() ) );
      
      strategyCombo = new JComboBox( LoadStrategyRegistry.getInstance().getStrategies() );
      UISupport.setPreferredHeight( strategyCombo, 18 );
		strategyCombo.setSelectedItem( loadTest.getLoadStrategy().getType());
		strategyCombo.addItemListener( new ItemListener() 
				{
					public void itemStateChanged(ItemEvent e)
					{
						Object item = e.getItem();
						if( item == null ) 
							return;
						
						setLoadStrategy(item.toString());
					}} );
      
		toolbar.add( runButton );
		toolbar.add( cancelButton );
		toolbar.add( statisticsGraphButton );
		toolbar.add( testTimesGraphButton );
		toolbar.add( resetButton );
		toolbar.add( exportButton );
     
		toolbar.add( optionsButton );
		toolbar.add( UISupport.createToolbarButton( new ShowOnlineHelpAction( HelpUrls.LOADTESTEDITOR_HELP_URL )));
		toolbar.add( Box.createHorizontalGlue() );
		buildLimitBar( toolbar );
      toolbar.addSeparator();
		
		progressBar = new JProgressBar( 0, 100 );
		progressBar.setPreferredSize( new Dimension( 70, 20 ));

		toolbar.addFixed( progressBar );
		
		ButtonBarBuilder builder = new ButtonBarBuilder();
		
		builder.addFixed( new JLabel( "Threads:" ));
      builder.addRelatedGap();
		
		threadsSpinner = new JSpinner( new SpinnerNumberModel(getModelItem().getThreadCount(), 1, 9999, 1) );
		UISupport.setPreferredHeight( threadsSpinner, 18 );
      threadsSpinner.getModel().addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				getModelItem().setThreadCount( ((SpinnerNumberModel) threadsSpinner.getModel()).getNumber().intValue() );
			}} );
      
		builder.addFixed( threadsSpinner);
		builder.addUnrelatedGap();
		
		LoadStrategy loadStrategy = loadTest.getLoadStrategy();
		
		builder.addFixed( new JLabel( "Strategy" ));
      builder.addRelatedGap();
      builder.addFixed( strategyCombo );
      builder.addUnrelatedGap();
		
      loadStrategyConfigurationPanel = new JPanel( new BorderLayout() );
		loadStrategyConfigurationPanel.add( loadStrategy.getConfigurationPanel(), BorderLayout.CENTER );
      
      builder.addFixed( loadStrategyConfigurationPanel );
      builder.setBorder( BorderFactory.createEmptyBorder( 2, 3, 3, 3 ) );
      
		return UISupport.buildPanelWithToolbar( toolbar, builder.getPanel() );
	}
	
	public void buildLimitBar( JXToolBar toolbar )
	{
		limitSpinnerModel = new SpinnerNumberModel(getModelItem().getTestLimit(), 0, Long.MAX_VALUE, 100 );
		
		limitSpinner = new JSpinner( limitSpinnerModel );
		limitSpinner.setPreferredSize( new Dimension( 70, 20 ));
		limitSpinner.setToolTipText( "Sets the limit for this test; requests per thread or seconds" );
		limitSpinner.getModel().addChangeListener( new ChangeListener() {

			public void stateChanged(ChangeEvent e)
			{
				int intValue = ((SpinnerNumberModel) limitSpinner.getModel()).getNumber().intValue();
				getModelItem().setTestLimit( intValue );
			}} );
		
		toolbar.addSeparator();
		toolbar.addFixed( new JLabel( "Limit:" ));
		toolbar.addSeparator();
		toolbar.addFixed(limitSpinner );
		toolbar.addSeparator();
		
		limitTypeCombo = new JComboBox( new String[] { "Runs", "Seconds"});
		
		if( getModelItem().getLimitType() == LoadTestLimitTypesConfig.TIME )
			limitTypeCombo.setSelectedIndex( 1 );
		
		toolbar.addFixed( limitTypeCombo);
		toolbar.addSeparator();
		
		limitTypeCombo.addItemListener( new ItemListener() 
		{
			public void itemStateChanged(ItemEvent e)
			{
				Object item = e.getItem();
				if( "Runs".equals( item ))
				{
					getModelItem().setLimitType( LoadTestLimitTypesConfig.COUNT );
				}
				else if( "Seconds".equals( item ))
				{
					getModelItem().setLimitType( LoadTestLimitTypesConfig.TIME );
				}
			}} );
	}

	public boolean onClose( boolean canCancel )
	{
		if( runner != null && runner.getStatus() == Status.RUNNING )
		{
			if( !UISupport.confirm( "Running test will be canceled when closing window. Close anyway?", "Close LoadTest" ))
				return false;
		}
		
		getModelItem().removeLoadTestRunListener( internalLoadTestListener );
		getModelItem().removePropertyChangeListener( WsdlLoadTest.THREADCOUNT_PROPERTY, this );
		
		if( runner != null && runner.getStatus() == Status.RUNNING )
			runner.cancel( "closing window" );
		
		if( statisticsDesktopPanel != null )
			SoapUI.getDesktop().closeDesktopPanel( statisticsDesktopPanel );
		
		if( statisticsHistoryDesktopPanel != null )
			SoapUI.getDesktop().closeDesktopPanel( statisticsHistoryDesktopPanel );
		
		assertionsTable.release();
		
		SoapUI.getDesktop().removeDesktopListener( desktopListener );
		
		return true;
	}

	public JComponent getComponent()
	{
		return contentPanel;
	}

   private final class InternalDesktopListener extends DesktopListenerAdapter
	{
		public void desktopPanelClosed(DesktopPanel desktopPanel)
		{
			if( desktopPanel == statisticsDesktopPanel )
				statisticsDesktopPanel = null;
			else if( desktopPanel == statisticsHistoryDesktopPanel )
				statisticsHistoryDesktopPanel = null;
		}
	}

	public class RunLoadTestAction extends AbstractAction
   {
		public RunLoadTestAction()
      {
         putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/run_testcase.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Runs this LoadTest" );
      }

		public void actionPerformed(ActionEvent e)
		{
			if( getModelItem().getTestCase().getTestStepCount() == 0 )
			{
				UISupport.showErrorMessage( "Missing TestSteps for testing!");
				return;
			}
			
			getModelItem().getStatisticsModel().reset();
			loadTestLog.clear();
			progressBar.setValue( 0 );
			
         runner = (WsdlLoadTestRunner) getModelItem().run();
         runButton.setEnabled( false );
		}
   }

   public class ResetAction extends AbstractAction
   {
		public ResetAction()
      {
         putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/reset_loadtest_statistics.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Resets statistics for this LoadTest" );
      }

		public void actionPerformed(ActionEvent e)
		{
			getModelItem().getStatisticsModel().reset();
		}
   }
   
   public class ShowStatisticsGraphAction extends AbstractAction
   {
		public ShowStatisticsGraphAction()
      {
         putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/stats_graph.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Shows the statistics graph" );
      }

		public void actionPerformed(ActionEvent e)
		{
			if( statisticsDesktopPanel == null )
				statisticsDesktopPanel = new StatisticsDesktopPanel( getModelItem() );
			
         UISupport.showDesktopPanel( statisticsDesktopPanel );
		}
   }

   public class ShowTestTimesGraphAction extends AbstractAction
   {
		public ShowTestTimesGraphAction()
      {
         putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/samples_graph.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Shows the Statistics History graph" );
      }

		public void actionPerformed(ActionEvent e)
		{
			if( statisticsHistoryDesktopPanel == null )
				statisticsHistoryDesktopPanel = new StatisticsHistoryDesktopPanel( getModelItem() );
			
         UISupport.showDesktopPanel( statisticsHistoryDesktopPanel );
		}
   }
   
   public class CancelRunTestCaseAction extends AbstractAction
   {

		public CancelRunTestCaseAction()
      {
         putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/stop_testcase.gif"));
         putValue( Action.SHORT_DESCRIPTION, "Stops running this LoadTest" );
      }
      
		public void actionPerformed(ActionEvent e)
		{
      	if( runner != null )
      	{
      		runner.cancel( "Canceled" );
      	}
      	
      	cancelButton.setEnabled( false );
      }
   }

   public boolean dependsOn(ModelItem modelItem)
	{
   	WsdlLoadTest loadTest = getModelItem();
   	
		return modelItem == loadTest || modelItem == loadTest.getTestCase() || 
			modelItem == loadTest.getTestCase().getTestSuite() || 
			modelItem == loadTest.getTestCase().getTestSuite().getProject();
	}
	
	public void setLoadStrategy(String type)
	{
		LoadStrategyFactory factory = LoadStrategyRegistry.getInstance().getFactory( type );
		LoadStrategy loadStrategy = factory.create();
		getModelItem().setLoadStrategy( loadStrategy );
		loadStrategyConfigurationPanel.removeAll();
		loadStrategyConfigurationPanel.add( loadStrategy.getConfigurationPanel(), BorderLayout.CENTER );
		loadStrategyConfigurationPanel.revalidate();
	}

	private class InternalLoadTestListener extends LoadTestRunListenerAdapter
	{
		public void beforeLoadTest( LoadTestRunner testRunner, LoadTestRunContext context )
		{
         loadTestLog.clear();
			
			loadTestStartTime = System.currentTimeMillis();
			loadTestIsRunning = true;
			if( getModelItem().getTestLimit() > 0 )
			{
				progressBar.setValue(0);
				progressBar.setString( null );
			}
			else
			{
				progressBar.setString("...");
			}
			
			progressBar.setStringPainted(true);

			runButton.setEnabled( false );
			cancelButton.setEnabled( true );
			strategyCombo.setEnabled( false );
			limitTypeCombo.setEnabled( false );
			threadsSpinner.setEnabled( getModelItem().getLoadStrategy().allowThreadCountChangeDuringRun() );
			
			new Thread( new ProgressBarUpdater() ).start();
		}

		public void afterLoadTest( LoadTestRunner testRunner, LoadTestRunContext context )
		{
			runButton.setEnabled( true );
			
			cancelButton.setEnabled( false );
			strategyCombo.setEnabled( true );
			limitTypeCombo.setEnabled( true );
			threadsSpinner.setEnabled( true );
			
			runner = null;
			loadTestIsRunning = false;
			
			if( progressBar.isIndeterminate() )
			{
				progressBar.setIndeterminate( false );
				progressBar.setValue(0);
			}
			else if( testRunner.getStatus() == Status.FINISHED )
			{
				progressBar.setValue( 100 );
			}
			
			if( testRunner.getStatus() == Status.FAILED )
			{
				UISupport.showErrorMessage( "LoadTest failed" );
			}
		}
		
		
	}

	public String getDescription()
	{
		return "LoadTest: [" + getModelItem().getName() + "] for TestCase [" + getModelItem().getTestCase().getName() + 
					"] in TestSuite [" + getModelItem().getTestCase().getTestSuite().getName() + "]";
	}
	
	private class ProgressBarUpdater implements Runnable
	{
		public void run()
		{
			while( true )
			{
				if( !loadTestIsRunning )
					break;

				if( getModelItem().getTestLimit() == 0 )
				{
					if( loadTestIsRunning && !progressBar.isIndeterminate())
					{
						progressBar.setIndeterminate( true );
						progressBar.setString("...");
					}
				}
				else if( getModelItem().getLimitType() == LoadTestLimitTypesConfig.TIME )
				{
					if( loadTestIsRunning && progressBar.isIndeterminate())
					{
						progressBar.setIndeterminate( false );
						progressBar.setString(null);
					}
					
					long timePassed = System.currentTimeMillis()-loadTestStartTime;
					int value = (int) ((timePassed*100)/(getModelItem().getTestLimit()*1000));
					progressBar.setValue( value );
				}
				else if( getModelItem().getLimitType() == LoadTestLimitTypesConfig.COUNT )
				{
					if( loadTestIsRunning && progressBar.isIndeterminate())
					{
						progressBar.setIndeterminate( false );
						progressBar.setString(null);
					}
					
					long counts = getModelItem().getStatisticsModel().getStatistic( LoadTestStatistics.TOTAL, Statistic.COUNT );
					if( counts > 0 )
						progressBar.setValue(  (int) ((counts*100)/getModelItem().getTestLimit()) );
				}

				try
				{
					Thread.sleep( 500 );
				}
				catch (InterruptedException e)
				{
					e.printStackTrace();
				}
			}
		}
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		if( evt.getPropertyName().equals( WsdlLoadTest.THREADCOUNT_PROPERTY ))
		{
			threadsSpinner.setValue( evt.getNewValue() );
		}
	}
	
}
