/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.request.components.editor.support;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.util.HashSet;
import java.util.Set;

import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlDocument;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlEditor;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlEditorView;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlLocation;
import com.eviware.soapui.impl.wsdl.panels.request.components.editor.XmlLocationListener;

public abstract class AbstractEditorView implements XmlEditorView, PropertyChangeListener
{
	private String title;
	private boolean isActive;
	private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport( this );
	private XmlDocument xmlDocument;
	private boolean xmlChanged;
	private Set<XmlLocationListener> listeners = new HashSet<XmlLocationListener>();
	private XmlEditor editor;
	
	public AbstractEditorView(String title)
	{
		super();
		this.title = title;
		xmlChanged = true;
	}

	public boolean activate(XmlLocation location)
	{
		isActive = true;
		if( xmlChanged  )
		{
			setXml( xmlDocument == null ? null : xmlDocument.getXml() );
		}
		
		return true;
	}

	public boolean deactivate()
	{
		isActive = false;
		xmlChanged = false;
		
		return true;
	}
	
	public boolean isActive()
	{
		return isActive;
	}

	public String getTitle()
	{
		return title;
	}

	public void setTitle( String title )
	{
		String oldTitle = this.title;
		this.title = title;
		
		propertyChangeSupport.firePropertyChange( TITLE_PROPERTY, oldTitle, title );
	}
	
	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
	   propertyChangeSupport.addPropertyChangeListener(propertyName, listener);	
	}

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		propertyChangeSupport.addPropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		propertyChangeSupport.removePropertyChangeListener( listener );
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		propertyChangeSupport.removePropertyChangeListener( propertyName, listener );
	}
	
	public XmlDocument getXmlDocument()
	{
		return xmlDocument;
	}

	public void setXmlDocument(XmlDocument xmlDocument)
	{
		if( this.xmlDocument != null )
		{
			this.xmlDocument.removePropertyChangeListener( XmlDocument.XML_PROPERTY, this );
		}
		
		this.xmlDocument = xmlDocument;
		
		if( xmlDocument != null )
		{
			this.xmlDocument.addPropertyChangeListener( XmlDocument.XML_PROPERTY, this );
//			if( isActive() )
				setXml( xmlDocument.getXml() );
		}
		else
		{
			setXml( null );
		}
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		if( isActive() )
			setXml( (String) evt.getNewValue() );
		else
			xmlChanged = true;
	}
	
	protected abstract void setXml( String xml );

	public void release()
	{
		if( this.xmlDocument != null )
		{
			this.xmlDocument.removePropertyChangeListener( XmlDocument.XML_PROPERTY, this );
		}		
	}

	public void addLocationListener(XmlLocationListener listener)
	{
		listeners.add( listener );
	}

	public void removeLocationListener(XmlLocationListener listener)
	{
		listeners.remove( listener );
	}
	
	public void fireLocationChanged( XmlLocation location )
	{
		for( XmlLocationListener listener : listeners )
			listener.locationChanged( location );
	}
	
	public XmlLocation getLocation()
	{
		return null;
	}

	public void setLocation(XmlLocation location)
	{
	}
	
	public void locationChanged(XmlLocation location)
	{
	}

	public void init( XmlEditor editor )
	{
		this.editor = editor;
	}

	public XmlEditor getEditor()
	{
		return editor;
	}

	public void requestFocus()
	{
	}
}
