/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractListModel;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.ListCellRenderer;
import javax.swing.SwingUtilities;
import javax.swing.event.PopupMenuEvent;
import javax.swing.event.PopupMenuListener;

import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.AssertionsListener;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlMessageAssertion;
import com.eviware.soapui.impl.wsdl.teststeps.actions.AddAssertionAction;
import com.eviware.soapui.impl.wsdl.teststeps.assertions.AssertionError;
import com.eviware.soapui.support.action.ActionSupport;

public class AssertionsPanel extends JPanel
{
	private AssertionListModel assertionListModel;
	private JList assertionList;
	private JPopupMenu assertionListPopup;
	private final Assertable assertable;

	public AssertionsPanel( Assertable assertable )
	{
		super( new BorderLayout() );
		this.assertable = assertable;
		
      assertionListModel = new AssertionListModel();
      assertionList = new JList( assertionListModel );
      assertionList.setToolTipText( "Assertions for this request" );
      assertionList.setCellRenderer( new AssertionCellRenderer() );
      
      assertionListPopup = new JPopupMenu();
      assertionListPopup.add( new AddAssertionAction( assertable ));
      
      assertionListPopup.addPopupMenuListener( new PopupMenuListener(){

         public void popupMenuWillBecomeVisible(PopupMenuEvent e)
         {
            while( assertionListPopup.getComponentCount() > 1 ) 
               assertionListPopup.remove( 1 );
            
            if( assertionListModel.getSize() == 0 ) return;
            int ix = assertionList.getSelectedIndex();
            if( ix == -1 ) return;
            
            WsdlMessageAssertion assertion = assertionListModel.getAssertionAt( ix );
            ActionSupport.addActions( assertion.getActions(), assertionListPopup );
         }

         public void popupMenuWillBecomeInvisible(PopupMenuEvent e)
         {
         }

         public void popupMenuCanceled(PopupMenuEvent e)
         {
         }});
      
      assertionList.setComponentPopupMenu( assertionListPopup );
      
      assertionList.addMouseListener( new MouseAdapter() {

			public void mouseClicked(MouseEvent e)
         {
            if( e.getClickCount() < 2 ) return;

            int ix = assertionList.getSelectedIndex();
            if( ix == -1 ) return;
            
            Object obj = assertionList.getModel().getElementAt( ix );
            if( obj instanceof WsdlMessageAssertion )
            {
	            WsdlMessageAssertion assertion = (WsdlMessageAssertion) obj;
	            if( assertion.isConfigurable() )
	               assertion.configure();
	            
	            return;
            }
            
   		   if( obj instanceof AssertionError )
   		   {
   		   	AssertionError error = (AssertionError) obj;
   		   	if( error.getLineNumber() >= 0 )
   		   	{
   		   		selectError(error);
   		   	}
   		   	else Toolkit.getDefaultToolkit().beep();
   		   }
   		   else Toolkit.getDefaultToolkit().beep();
         }});
      
      add( new JScrollPane( assertionList ), BorderLayout.CENTER );
   }
	
	public void setEnabled( boolean enabled )
	{
		assertionList.setEnabled( enabled );
	}

   protected void selectError(AssertionError error)
	{
	}
   
	private static class AssertionCellRenderer extends JLabel implements ListCellRenderer 
   {
      public Component getListCellRendererComponent(
        JList list,
        Object value,           
        int index,               
        boolean isSelected,      
        boolean cellHasFocus)    
      {
      	if( value instanceof WsdlMessageAssertion )
      	{
      		WsdlMessageAssertion assertion = (WsdlMessageAssertion) value;
      		setText( assertion.getName() + " - " + assertion.getStatus().toString() );
            setIcon( assertion.getIcon() );
      	}
      	else if( value instanceof AssertionError )
      	{
            AssertionError assertion = (AssertionError) value;
            setText( " -> " + assertion.toString() );
            setIcon( null );
      	}
      	else if( value instanceof String )
      	{
      		setText( value.toString() );
      	}
      	
          if (isSelected) 
          {
             setBackground(list.getSelectionBackground());
             setForeground(list.getSelectionForeground());
          }
          else 
          {
             setBackground(list.getBackground());
             setForeground(list.getForeground());
          }
          
          setEnabled(list.isEnabled());
          setFont(list.getFont());
          setOpaque(true);
          
          return this;
      }
  }
   
   private class AssertionListModel extends AbstractListModel implements PropertyChangeListener, 
       AssertionsListener
   {
   	private List<Object> items = new ArrayList<Object>();
   	
      public AssertionListModel()
      {
      	init();
      }
      
      public int getSize()
      {
      	return items.size();
      }

      public Object getElementAt(int index)
      {
      	return index >= items.size() ? null : items.get( index );
      }

      public WsdlMessageAssertion getAssertionAt( int index )
      {
      	Object object = items.get( index );
      	while( object instanceof AssertionError && index > 0 )
      	{
      		object = items.get( --index );
      	}
      	
      	return (WsdlMessageAssertion) object;
      }
      
      public void refresh()
      {
      	synchronized( this )
			{
				release();
	      	init();
	         fireContentsChanged( this, 0, getSize()-1 );
			}
      }

		private void init()
		{
      	assertable.addAssertionsListener( this );
			
         for( int c = 0; c < assertable.getAssertionCount(); c++ )
         {
         	WsdlMessageAssertion assertion = assertable.getAssertionAt( c );
         	addAssertion(assertion);
         }
		}
		
		public void release()
		{
         items.clear();

         for( int c = 0; c < assertable.getAssertionCount(); c++ )
         {
         	WsdlMessageAssertion assertion = assertable.getAssertionAt( c );
         	assertion.removePropertyChangeListener( this );
         }
         
         assertable.removeAssertionsListener( this );
		}

		public synchronized void propertyChange(PropertyChangeEvent evt)
		{
			if( SwingUtilities.isEventDispatchThread() )
				refresh();
			else SwingUtilities.invokeLater( new Runnable() {

				public void run()
				{
					refresh();
				}} );			
		}

		public void assertionAdded(WsdlMessageAssertion assertion)
		{
			synchronized( this )
			{
				int sz = getSize();
				addAssertion(assertion);
      	
				fireIntervalAdded( this, sz, items.size()-1 );
			}
		}

		private void addAssertion(WsdlMessageAssertion assertion)
		{
			assertion.addPropertyChangeListener( this );
      	items.add( assertion );
      	
      	AssertionError[] errors = assertion.getErrors();
      	if( errors != null)
      	{
      		for( int i = 0; i < errors.length; i++ )
      		items.add( errors[i] );
      	}
		}

		public void assertionRemoved(WsdlMessageAssertion assertion)
		{
			synchronized( this )
			{
				int ix = items.indexOf( assertion );
				if( ix == -1 ) return;
	
				assertion.removePropertyChangeListener( this );
				items.remove( ix );
				fireIntervalRemoved( this, ix, ix );
				
				// remove associated errors
				while( ix < items.size() && items.get( ix ) instanceof AssertionError )
				{
					items.remove( ix );
					fireIntervalRemoved( this, ix, ix );
				}
			}
		}
   }

	public void release()
	{
		assertionListModel.release();
	}
}
