/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.panels.teststeps;

import javax.swing.JPanel;

import com.eviware.soapui.impl.EmptyPanelBuilder;
import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequest;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestRequestStep;
import com.eviware.soapui.support.components.JPropertiesTable;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * PanelBuilder for WsdlTestRequest
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRequestPanelBuilder extends EmptyPanelBuilder<WsdlTestRequestStep>
{
   public WsdlTestRequestPanelBuilder()
   {
   }

   public DesktopPanel buildDesktopPanel(WsdlTestRequestStep testStep)
   {
      return new WsdlTestRequestDesktopPanel(testStep);
   }

   public boolean hasDesktopPanel()
   {
      return true;
   }
   
   public JPanel buildOverviewPanel(WsdlTestRequestStep testStep)
   {
   	WsdlTestRequest request = testStep.getTestRequest();
   	JPropertiesTable<WsdlTestRequest> table = new JPropertiesTable<WsdlTestRequest>( "TestRequest Properties" );
   	request.addPropertyChangeListener( table.getTableModel() );
   	
   	table.addProperty( "Name", "name" );
   	table.addProperty( "Encoding", "encoding", new String[] {null, "UTF-8", "iso-8859-1" } );
   	table.addProperty( "Endpoint", "endpoint" );
   	table.addProperty( "Enable MTOM/Inline", "mtomEnabled", JPropertiesTable.BOOLEAN_OPTIONS );
   	table.addProperty( "Username", "username", true );
   	table.addProperty( "Password", "password", true );
   	table.addProperty( "Domain", "domain", true );
   	table.addProperty( "WSS-Password Type", "wssPasswordType", 
   			new String[] {WsdlRequest.PW_TYPE_NONE, WsdlRequest.PW_TYPE_TEXT, WsdlRequest.PW_TYPE_DIGEST});
   	table.addProperty( "Inline Response Attachments", "inlineResponseAttachments", 
   			JPropertiesTable.BOOLEAN_OPTIONS );
   	table.addProperty( "Expand MTOM Attachments", "expandMtomResponseAttachments", 
	   			JPropertiesTable.BOOLEAN_OPTIONS );
   	table.addProperty( "Strip whitspaces", "stripWhitespaces", JPropertiesTable.BOOLEAN_OPTIONS );
   	table.addProperty( "Disable multiparts", "multipartEnabled", JPropertiesTable.BOOLEAN_OPTIONS );
   	
   	table.setPropertyObject( request );
   	
   	return table;
   }

   public boolean hasOverviewPanel()
   {
      return true;
   }
}
