/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.submit.filters;

import java.io.StringReader;
import java.io.StringWriter;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.ws.security.WSConstants;
import org.apache.ws.security.message.WSSecHeader;
import org.apache.ws.security.message.WSSecUsernameToken;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;

import com.eviware.soapui.impl.wsdl.WsdlRequest;
import com.eviware.soapui.impl.wsdl.submit.RequestFilter;
import com.eviware.soapui.impl.wsdl.submit.transports.http.BaseHttpRequestTransport;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.support.xml.XmlUtils;

/**
 * Modifies the request message to include WS-Securty Username tokens
 * 
 * @author Ole.Matzura
 */

public class WsSecurityAuthenticationRequestFilter implements RequestFilter
{
	private static DocumentBuilderFactory dbf;
	private static DocumentBuilder db;

	static
	{
		dbf = DocumentBuilderFactory.newInstance();
		dbf.setValidating(false);
	   dbf.setNamespaceAware(true);
	   
	   try
		{
			db = dbf.newDocumentBuilder();
		}
		catch (ParserConfigurationException e)
		{
			e.printStackTrace();
		}
	}

	public void filterRequest(SubmitContext context, WsdlRequest wsdlRequest)
	{
		try 
      {
         String pwType = wsdlRequest.getWssPasswordType();
         if (pwType == null || pwType.length() == 0 || WsdlRequest.PW_TYPE_NONE.equals(pwType)) 
         {
            return ;
         }

         String request = (String) context.getProperty( BaseHttpRequestTransport.REQUEST_CONTENT );
			Document doc = db.parse(new InputSource( new StringReader( request )));

			WSSecUsernameToken wsa = new WSSecUsernameToken();
         if (WsdlRequest.PW_TYPE_DIGEST.equals(pwType)) 
         {
            wsa.setPasswordType(WSConstants.PASSWORD_DIGEST);
         } 
         else 
         {
            wsa.setPasswordType(WSConstants.PASSWORD_TEXT);
         }
         
         wsa.setUserInfo(wsdlRequest.getUsername(), wsdlRequest.getPassword() ); 
         
         WSSecHeader secHeader = new WSSecHeader();
         secHeader.insertSecurityHeader( doc );
			wsa.build(doc, secHeader );

         StringWriter writer = new StringWriter();
 			XmlUtils.serializePretty( doc, writer );
 			context.setProperty( BaseHttpRequestTransport.REQUEST_CONTENT, writer.toString() );
      } 
		catch (Throwable e) 
		{
          e.printStackTrace();
      }
	}
}
