/*
 *  soapUI, copyright (C) 2005 Ole Matzura / eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.support.http;

import java.net.MalformedURLException;
import java.net.URL;

import org.apache.commons.httpclient.Credentials;
import org.apache.commons.httpclient.HostConfiguration;
import org.apache.commons.httpclient.HttpState;
import org.apache.commons.httpclient.UsernamePasswordCredentials;
import org.apache.commons.httpclient.auth.AuthScope;

import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.settings.ProxySettings;

public class ProxyUtils
{
	public static HostConfiguration initProxySettings( Settings settings, HttpState httpState, HostConfiguration hostConfiguration, 
				String urlString )
	{
		//	init proxy settings
		if( hostConfiguration == null )
			hostConfiguration = new HostConfiguration();
		
		// check system properties first
		String proxyHost = System.getProperty( "http.proxyHost" );
	   String proxyPort = System.getProperty( "http.proxyPort" );
	   
	   if( proxyHost == null )
	   	proxyHost = settings.getString(ProxySettings.HOST, null);
	   
	   if( proxyPort == null )
	   	proxyPort = settings.getString(ProxySettings.PORT, null);
	
		if (proxyHost != null && proxyHost.length() > 0 && proxyPort != null && proxyPort.length() > 0)
		{
			// check excludes
			String[] excludes = settings.getString(ProxySettings.EXCLUDES, "").split( "," );
			
			try
			{
				URL url = new URL( urlString );
				
				if( !excludes( excludes, url.getHost(), url.getPort() ) )
				{
					hostConfiguration.setProxy(proxyHost, Integer.parseInt(proxyPort));
		
					String proxyUsername = settings.getString(ProxySettings.USERNAME, null);
					String proxyPassword = settings.getString(ProxySettings.PASSWORD, null);
		
					if (proxyUsername != null && proxyPassword != null )
					{
						Credentials defaultcreds = new UsernamePasswordCredentials(proxyUsername, proxyPassword);
						httpState.setProxyCredentials(AuthScope.ANY, defaultcreds);
					}
				}
			}
			catch( MalformedURLException e )
			{
				e.printStackTrace();
			}
			
		}
		
		return hostConfiguration;
	}
	
	public static boolean excludes( String [] excludes, String proxyHost, int proxyPort )
	{
		for( int c = 0; c < excludes.length; c++ )
		{
			String exclude = excludes[c].trim();
			if( exclude.length() == 0 )
				continue;
			
			// check for port
			int ix = exclude.indexOf( ':' );
			
			if( ix >= 0 && exclude.length() > ix+1 )
			{
				String excludePort = exclude.substring( ix+1 );
				if( proxyPort != -1 && excludePort.equals( String.valueOf( proxyPort )))
				{
					exclude = exclude.substring( 0, ix );
				}
				else
				{
					continue;
				}
			}
			
			if( proxyHost.endsWith( exclude ))
				return true;
		}
		
		return false;
	}
}
