/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.testcase;

import com.eviware.soapui.model.support.AbstractSubmitContext;
import com.eviware.soapui.model.support.PropertiesMap;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestRunContext;
import com.eviware.soapui.model.testsuite.TestRunner;
import com.eviware.soapui.model.testsuite.TestStep;

/**
 * TestRunContext for WsdlTestCase runners
 * 
 * @author Ole.Matzura
 */

public class WsdlTestRunContext extends AbstractSubmitContext implements TestRunContext
{
	private final TestRunner testRunner;
	private int currentStepIndex;
	private TestCase testCase;

	public WsdlTestRunContext( TestRunner testRunner, PropertiesMap properties )
	{
		super( properties );
		this.testRunner = testRunner;
	}

	public WsdlTestRunContext( TestStep testStep )
	{
		this( null, null );
		
		testCase = testStep.getTestCase();
		currentStepIndex = testCase.getIndexOfTestStep( testStep );
	}

	public TestStep getCurrentStep()
	{
		if( currentStepIndex < 0 || currentStepIndex >= getTestCase().getTestStepCount() )
			return null;
		
		return getTestCase().getTestStepAt( currentStepIndex );
	}

	public int getCurrentStepIndex()
	{
		return currentStepIndex;
	}
	
	public void setCurrentStep( int index )
	{
		currentStepIndex = index;
	}

	public TestRunner getTestRunner()
	{
		return testRunner;
	}

	public Object getProperty(String testStepName, String propertyName)
	{
		TestStep testStep = getTestCase().getTestStepByName( testStepName );
		return testStep == null ? null : testStep.getPropertyValue( propertyName );
	}

	public TestCase getTestCase()
	{
		return testRunner == null ? testCase : testRunner.getTestCase();
	}
	
	public Object getProperty(String name)
	{
		WsdlTestCase testCase = (WsdlTestCase) getTestCase();
		TestStep testStep = currentStepIndex >= 0 && currentStepIndex < testCase.getTestStepCount() ? 
				testCase.getTestStepAt( currentStepIndex ) : null;
		
		return getProperty( name, testStep, testCase);
	}

	public void reset()
	{
		resetProperties();
		currentStepIndex = 0;
	}
}
