/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps;

import java.io.PrintWriter;
import java.lang.ref.SoftReference;

import com.eviware.soapui.impl.wsdl.submit.transports.http.WsdlResponse;
import com.eviware.soapui.impl.wsdl.teststeps.actions.ShowMessageExchangeAction;
import com.eviware.soapui.model.iface.Attachment;
import com.eviware.soapui.model.iface.MessageExchange;
import com.eviware.soapui.model.iface.Response;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.XmlUtils;

public class WsdlTestRequestStepResult extends WsdlTestStepResult implements MessageExchange
{
	private SoftReference<String> requestContent;
	private SoftReference<WsdlResponse> response;
	private String domain;
	private String username;
	private String endpoint;
	private String encoding;
	private String password;
	private StringToStringMap properties;
	
	public WsdlTestRequestStepResult(WsdlTestRequestStep step )
	{
		super( step );
	}

	public String getRequestContent()
	{
		if( isDiscarded() )
			return "<discarded>";
		
		if( requestContent != null && requestContent.get() == null ) 
			return "<garbage-collected>";
		
		return requestContent == null ? null : requestContent.get();
	}

	public void setRequestContent(String requestContent)
	{
		this.requestContent = new SoftReference<String>(requestContent);
	}

	public WsdlResponse getResponse()
	{
		if( isDiscarded() )
			return null;
		
		return response == null ? null : response.get();
	}
	
	@Override
	public ActionList getActions()
	{
		ActionList actions = super.getActions();
		if( actions == null ) 
		{
			if( isDiscarded() )
				return actions;
			else 
				addAction( new ShowMessageExchangeAction( this, "TestStep" ), true );
			
			return super.getActions();
		}
		
		return actions;
	}

	public void setResponse(WsdlResponse response)
	{
		this.response = new SoftReference<WsdlResponse>(response);
	}

	public String getDomain()
	{
		return domain;
	}

	public void setDomain(String domain)
	{
		this.domain = domain;
		addProperty( "domain", domain );
	}

	private void addProperty( String key, String value )
	{
		if( properties == null )
			properties = new StringToStringMap();
		
		properties.put( key, value );
	}

	public String getEncoding()
	{
		return encoding;
	}

	public void setEncoding(String encoding)
	{
		this.encoding = encoding;
		addProperty( "encoding", encoding );
	}

	public String getEndpoint()
	{
		return endpoint;
	}

	public void setEndpoint(String endpoint)
	{
		this.endpoint = endpoint;
		addProperty( "endpoint", endpoint );
	}

	public String getPassword()
	{
		return password;
	}

	public void setPassword(String password)
	{
		this.password = password;
		addProperty( "password", password );
	}

	public String getUsername()
	{
		return username;
	}

	public void setUsername(String username)
	{
		this.username = username;
		addProperty( "username", username );
	}

	public void discard()
	{
		super.discard();
		
		requestContent = null;
		response = null;
		properties = null;
	}

	public void writeTo(PrintWriter writer)
	{
		super.writeTo( writer );
		
		writer.println( "----------------------------------------------------" );
		writer.println( "Encoding: " + getEncoding() );
		writer.println( "Endpoint: " + getEndpoint() );
		writer.println( "Username: " + getUsername() );
		writer.println( "Password: " + getPassword() );
		writer.println( "Domain: " + getDomain() );

		Response resp = response == null ? null : response.get();
		
		writer.println( "---------------- Request ---------------------------" );
		String request = requestContent == null ? null : requestContent.get();
		if( request != null )
			writer.println( XmlUtils.prettyPrintXml( request ) );
		else
			writer.println( "- missing request / garbage collected -" );

		if( resp != null )
		{
			writer.println( "Request Headers: " + resp.getRequestHeaders().toString() );
		}
		
		writer.println( "---------------- Response --------------------------" );
		if( resp != null )
		{
			String respContent = resp.getContentAsString();
			if( respContent != null )
				writer.println( XmlUtils.prettyPrintXml( respContent ) );

			writer.println( "Response Headers: " + resp.getResponseHeaders().toString() );
		}
		else
			writer.println( "- missing response / garbage collected -" );
	}

	public StringToStringMap getProperties()
	{
		return properties;
	}

	public Attachment[] getRequestAttachments()
	{
		if( isDiscarded() || response == null|| response.get() == null )
			return null;
		
		return response.get().getRequest().getAttachments();
	}

	public StringToStringMap getRequestHeaders()
	{
		if( isDiscarded() || response == null || response.get() == null )
			return null;
		
		return response.get().getRequestHeaders();
	}

	public Attachment[] getResponseAttachments()
	{
		if( isDiscarded() || response == null || response.get() == null )
			return null;
		
		return response.get().getAttachments();
	}

	public String getResponseContent()
	{
		if( isDiscarded() )
			return "<discarded>";
		
		if( response == null )
			return "<missing response>";
		
		if( response != null && response.get() == null ) 
			return "<garbage-collected>";
		
		return response.get().getContentAsString();
	}

	public StringToStringMap getResponseHeaders()
	{
		if( isDiscarded() || response == null || response.get() == null )
			return null;
		
		return response.get().getResponseHeaders();
	}

	public long getTimestamp()
	{
		if( isDiscarded() || response == null || response.get() == null )
			return -1;
		
		return response.get().getTimestamp();
	}
}