/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.SoapUI;
import com.eviware.soapui.impl.wsdl.testcase.WsdlTestCase;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlTestStep;
import com.eviware.soapui.model.project.Project;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.support.UISupport;

/**
 * Clones a WsdlTestStep
 * 
 * @author Ole.Matzura
 */

public class CloneTestStepAction extends AbstractAction
{
   private final WsdlTestStep testStep;
	private final String typeName;

	public CloneTestStepAction( WsdlTestStep testStep, String typeName )
   {
      super( "Clone" );
		this.testStep = testStep;
		this.typeName = typeName;
      putValue( Action.SHORT_DESCRIPTION, "Creates a copy of this " + typeName );
      putValue( Action.SMALL_ICON, UISupport.createImageIcon( "/clone_request.gif"));
      putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( "F9" ));
   }
   
   public void actionPerformed(ActionEvent e)
	{
      WsdlTestCase testCase = (WsdlTestCase) testStep.getTestCase();
      Project project = testCase.getTestSuite().getProject();

      List<WsdlTestCase> testCases = new ArrayList<WsdlTestCase>();
      List<String> options = new ArrayList<String>();
      
      for( int c = 0; c < project.getTestSuiteCount(); c++ )
      {
      	TestSuite testSuite = project.getTestSuiteAt(c );
      	for( int i = 0; i < testSuite.getTestCaseCount(); i++ )
      	{
      		testCase = (WsdlTestCase) testSuite.getTestCaseAt( i );
      		testCases.add( testCase );
      		options.add( testSuite.getName() + " : " + testCase.getName() );
      	}
      }

      if( options.size() > 1 )
      {
         String testCaseName = (String) UISupport.prompt( "Select TestCase to clone to", "Clone " + typeName, options.toArray(),
         		testStep.getTestCase().getTestSuite().getName() + " : " + testStep.getTestCase().getName());
         if( testCaseName == null )
         	return;
         
         testCase = testCases.get( options.indexOf( testCaseName ));
      }
      
      if( SoapUI.getTestMonitor().hasRunningTest( testCase ))
      {
      	UISupport.showErrorMessage( "Cannot clone to running TestCase" );
      	return;
      }
      
      String name = UISupport.prompt( "Specify name of cloned " + typeName, "Clone " + typeName,
      		"Copy of " + testStep.getName() );
      if( name == null ) return;
      
      WsdlTestStep newStep = (WsdlTestStep) testCase.cloneStep( testStep, name );
      
      UISupport.selectAndShow( newStep );
   }

}
