/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.impl.wsdl.teststeps.assertions;

import org.apache.xmlbeans.XmlObject;

import com.eviware.soapui.config.RequestAssertionConfig;
import com.eviware.soapui.impl.wsdl.panels.support.assertions.Assertable;
import com.eviware.soapui.impl.wsdl.submit.WsdlMessageExchange;
import com.eviware.soapui.impl.wsdl.submit.filters.PropertyExpansionRequestFilter;
import com.eviware.soapui.impl.wsdl.support.HelpUrls;
import com.eviware.soapui.impl.wsdl.teststeps.WsdlMessageAssertion;
import com.eviware.soapui.model.iface.SubmitContext;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.types.StringToStringMap;
import com.eviware.soapui.support.xml.XmlObjectConfigurationBuilder;
import com.eviware.soapui.support.xml.XmlObjectConfigurationReader;
import com.eviware.x.form.XForm;
import com.eviware.x.form.XFormDialog;
import com.eviware.x.form.XFormDialogBuilder;
import com.eviware.x.form.XFormFactory;

/**
 * Assertion that checks for a specified text token in the associated
 * WsdlTestRequests response XML message
 * 
 * @author Ole.Matzura
 */

public class SimpleContainsAssertion extends WsdlMessageAssertion implements RequestAssertion,
			ResponseAssertion
{
	private String token;
	private XFormDialog dialog;
	private boolean ignoreCase;
	public static final String ID = "Simple Contains";
	private static final String CONTENT = "Content";
	private static final String IGNORE_CASE = "Ignore Case";

	public SimpleContainsAssertion( RequestAssertionConfig assertionConfig, Assertable assertable )
	{
		super( assertionConfig, assertable );

		XmlObjectConfigurationReader reader = new XmlObjectConfigurationReader( getConfiguration() );
		token = reader.readString( "token", null );
		ignoreCase = reader.readBoolean( "ignoreCase", false );
	}

	public String internalAssertResponse( WsdlMessageExchange messageExchange, SubmitContext context )
				throws AssertionException
	{
		return assertContent( context, messageExchange.getResponseContent(), "Response" );
	}

	private String assertContent( SubmitContext context, String content, String type ) throws AssertionException
	{
		if( token == null )
			token = "";
		String replToken = PropertyExpansionRequestFilter.expandProperties( context, token );

		if( replToken.length() > 0 )
		{
			int ix = ignoreCase ? content.toUpperCase().indexOf( replToken.toUpperCase() ) : content
						.indexOf( replToken );

			if( ix == -1 )
				throw new AssertionException( new AssertionError( "Missing token [" + replToken + "] in " + type ) );
		}
		
		return "Response contains token [" + replToken + "]";
	}

	public boolean configure()
	{
		if( dialog == null )
			buildDialog();

		StringToStringMap values = new StringToStringMap();
		values.put( CONTENT, token );
		values.put( IGNORE_CASE, ignoreCase );

		values = dialog.show( values );
		if( dialog.getReturnValue() == XFormDialog.OK_OPTION )
		{
			token = values.get( CONTENT );
			ignoreCase = values.getBoolean( IGNORE_CASE );
		}

		setConfiguration( createConfiguration() );
		return true;
	}

	protected XmlObject createConfiguration()
	{
		XmlObjectConfigurationBuilder builder = new XmlObjectConfigurationBuilder();
		builder.add( "token", token );
		builder.add( "ignoreCase", ignoreCase );
		return builder.finish();
	}

	public boolean isConfigurable()
	{
		return true;
	}

	private void buildDialog()
	{
		XFormDialogBuilder builder = XFormFactory.createDialogBuilder( "Simple Contains Assertion" );
		XForm mainForm = builder.createForm( "Basic" );

		mainForm.addTextField( CONTENT, "Content to check for", XForm.FieldType.TEXT ).setWidth( 20 );
		mainForm.addCheckBox( IGNORE_CASE, "Ignore case in comparison" );

		dialog = builder.buildDialog( builder
					.buildOkCancelHelpActions( HelpUrls.SIMPLE_CONTAINS_HELP_URL ), "Specify options",
					UISupport.OPTIONS_ICON );
	}

	@Override
	protected String internalAssertRequest( WsdlMessageExchange messageExchange,
				SubmitContext context ) throws AssertionException
	{
		return assertContent( context, messageExchange.getRequestContent(), "Request" );
	}
}
