/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.support;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.AbstractAction;
import javax.swing.Action;

import com.eviware.soapui.impl.EmptyPanelBuilder;
import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.support.UISupport;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.DefaultActionList;

/**
 * Base-class for ModelItem implementations
 * 
 * @author Ole.Matzura
 */

public abstract class AbstractModelItem implements ModelItem
{
	private DefaultActionList actions;
   private PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport( this );

   protected void addAction( Action action )
   {
   	addAction( action, false );
   }
   
   protected void addAction( Action action, boolean isDefault )
   {
   	if( actions == null )
			initActions();
   	
      actions.addAction( action );
      if( isDefault )
      	actions.setDefaultAction( action );
      
      notifyPropertyChanged( ModelItem.ACTIONS_PROPERTY, null, getActions() );
   }

	private void initActions()
	{
		actions = new DefaultActionList( getName() );
		actions.setDefaultAction( new ShowModelItemAction() );
	}
   
   public ActionList getActions()
   {
   	if( actions == null )
   		initActions();
   	
      return actions;
   }
   
   public PanelBuilder getPanelBuilder()
   {
      return EmptyPanelBuilder.get();
   }
   
   public String getDescription()
   {
      return getName();
   }

   public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
   {
      propertyChangeSupport.addPropertyChangeListener( propertyName, listener );
   }

   public void addPropertyChangeListener(PropertyChangeListener listener)
   {
      propertyChangeSupport.addPropertyChangeListener( listener );
   }

   public void removePropertyChangeListener(PropertyChangeListener listener)
   {
      propertyChangeSupport.removePropertyChangeListener( listener );
   }
   
   public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
   	propertyChangeSupport.removePropertyChangeListener( propertyName, listener );
	}

	protected void notifyPropertyChanged( String name, Object oldValue, Object newValue )
   {
      propertyChangeSupport.firePropertyChange( name, oldValue, newValue );
   }

   protected void notifyPropertyChanged( String name, int oldValue, int newValue )
   {
      propertyChangeSupport.firePropertyChange( name, oldValue, newValue );
   }

   protected void notifyPropertyChanged( String name, boolean oldValue, boolean newValue )
   {
      propertyChangeSupport.firePropertyChange( name, oldValue, newValue );
   }

	public class ShowModelItemAction extends AbstractAction
	{
		public ShowModelItemAction()
		{
			super("Show Editor");
		}

		public void actionPerformed(ActionEvent e)
		{
			UISupport.selectAndShow( AbstractModelItem.this );
		}
	}
}
