/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree;

import java.awt.Component;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JPopupMenu;

import com.eviware.soapui.model.ModelItem;
import com.eviware.soapui.model.PanelBuilder;
import com.eviware.soapui.model.settings.Settings;
import com.eviware.soapui.support.action.ActionList;
import com.eviware.soapui.support.action.ActionSupport;
import com.eviware.soapui.support.action.DefaultActionList;
import com.eviware.soapui.ui.desktop.DesktopPanel;

/**
 * Base implementation of SoapUITreeNode interface
 * 
 * @author Ole.Matzura
 */

public abstract class AbstractTreeNode implements SoapUITreeNode
{
	private DummyModelItem modelItem;
	private JPopupMenu popup;
	private ActionList actions;

	public AbstractTreeNode(String name, ImageIcon icon)
	{
		modelItem = new DummyModelItem( name, icon );
	}
	
	public boolean valueChanged(Object newValue)
	{
		return false;
	}

	public boolean isLeaf()
	{
		return getChildCount() == 0;
	}

	public JPopupMenu getPopup()
	{
		if( popup == null && actions != null )
			popup = ActionSupport.buildPopup( actions );
		
		return popup;
	}
	
	public void setLabel( String label )
	{
		modelItem.setLabel( label );
	}
	
	public ActionList getActions()
	{
		return actions;
	}
	
	public void addAction( Action action, boolean isDefault )
	{
		if( actions == null )
			actions = new DefaultActionList( modelItem.getName() );
		
		actions.addAction( action );
		if( isDefault && actions instanceof DefaultActionList )
			((DefaultActionList)actions).setDefaultAction( action );
	}

	public void setActions(ActionList actions)
	{
		this.actions = actions;
	}

	public Component getOverviewPanel()
	{
		return null;
	}

	public DesktopPanel getDesktopPanel()
	{
		return null;
	}

	public ModelItem getModelItem()
	{
		return modelItem;
	}

	public boolean hasContentPanel()
	{
		return false;
	}
	
	public class DummyModelItem implements ModelItem
	{
		private String name;
		private ImageIcon icon;
		private PropertyChangeSupport propertyChangeSupport;
		
		public DummyModelItem(String name, ImageIcon icon)
		{
			this.name = name;
			this.icon = icon;
		}

		public void setLabel( String label )
		{
			String oldName = this.name;
			this.name = label;
			
			if( propertyChangeSupport != null )
			{
				propertyChangeSupport.firePropertyChange( ModelItem.NAME_PROPERTY, oldName, label );
			}
		}

		public String getName()
		{
			return name;
		}

		public ImageIcon getIcon()
		{
			return icon;
		}

		public ActionList getActions()
		{
			return actions;
		}

		public PanelBuilder getPanelBuilder()
		{
			return null;
		}

		public String getDescription()
		{
			return name;
		}

		public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
		{
			if( propertyChangeSupport == null )
				propertyChangeSupport = new PropertyChangeSupport( this );
			
			propertyChangeSupport.addPropertyChangeListener( propertyName, listener );
		}

		public void addPropertyChangeListener(PropertyChangeListener listener)
		{
			if( propertyChangeSupport == null )
				propertyChangeSupport = new PropertyChangeSupport( this );
			
			propertyChangeSupport.addPropertyChangeListener( listener );
		}

		public void removePropertyChangeListener(PropertyChangeListener listener)
		{
			if( propertyChangeSupport != null )
				propertyChangeSupport.removePropertyChangeListener( listener );
		}

		public SoapUITreeNode getTreeNode()
		{
			return AbstractTreeNode.this;
		}

		public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
		{
			if( propertyChangeSupport != null )
				propertyChangeSupport.removePropertyChangeListener( propertyName, listener );
		}

		public Settings getSettings()
		{
			return null;
		}

		public void release()
		{
		}}


	public String toString()
	{
		return modelItem.getName();
	}
	
	public void reorder( boolean notify )
	{
	}
}
