/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.model.tree.nodes;

import java.util.ArrayList;
import java.util.List;

import com.eviware.soapui.model.testsuite.LoadTest;
import com.eviware.soapui.model.testsuite.TestCase;
import com.eviware.soapui.model.testsuite.TestStep;
import com.eviware.soapui.model.testsuite.TestSuite;
import com.eviware.soapui.model.testsuite.TestSuiteListener;
import com.eviware.soapui.model.tree.AbstractModelItemTreeNode;
import com.eviware.soapui.model.tree.SoapUITreeModel;
import com.eviware.soapui.model.tree.SoapUITreeNode;
import com.eviware.soapui.settings.UISettings;

/**
 * SoapUITreeNode for TestSuite implementations
 * 
 * @author Ole.Matzura
 */

public class TestSuiteTreeNode extends AbstractModelItemTreeNode<TestSuite>
{
	private InternalTestSuiteListener internalTestSuiteListener = new InternalTestSuiteListener();;
   private ReorderPropertyChangeListener propertyChangeListener = new ReorderPropertyChangeListener();
	private List<TestCaseTreeNode> testCaseNodes = new ArrayList<TestCaseTreeNode>();

	public TestSuiteTreeNode(TestSuite testSuite, SoapUITreeModel treeModel )
   {
      super( testSuite, testSuite.getProject(), treeModel );
      
		testSuite.addTestSuiteListener( internalTestSuiteListener );
		
		for( int c = 0; c < testSuite.getTestCaseCount(); c++ )
		{
			TestCase testCase = testSuite.getTestCaseAt( c );
			testCase.addPropertyChangeListener( TestCase.NAME_PROPERTY, propertyChangeListener );
			testCaseNodes.add( new TestCaseTreeNode( testCase, getTreeModel() ));
		}
		
		initOrdering( testCaseNodes, UISettings.ORDER_TESTCASES );
		
		getTreeModel().mapModelItems( testCaseNodes );
   }
	
	public void release()
	{
		super.release();
		
		getTestSuite().removeTestSuiteListener( internalTestSuiteListener );
		
		for( TestCaseTreeNode treeNode : testCaseNodes )
			treeNode.release();
	}
	
   public TestSuite getTestSuite()
   {
      return (TestSuite) getModelItem();
   }
   
   private class InternalTestSuiteListener implements TestSuiteListener
   {
      public void testCaseAdded(TestCase testCase)
      {
      	TestCaseTreeNode testCaseTreeNode = new TestCaseTreeNode( testCase, getTreeModel() );
      	testCaseNodes.add( testCaseTreeNode );
      	reorder( false );
      	
      	testCase.addPropertyChangeListener( TestCase.NAME_PROPERTY, propertyChangeListener );
      	getTreeModel().notifyNodeInserted( testCaseTreeNode );
      }

      public void testCaseRemoved(TestCase testCase)
      {
      	SoapUITreeNode treeNode = getTreeModel().getTreeNode( testCase );
      	if( testCaseNodes.contains( treeNode ))
      	{
      		getTreeModel().notifyNodeRemoved( treeNode );
      		testCaseNodes.remove( treeNode );
      		testCase.removePropertyChangeListener(  propertyChangeListener );
      	}
      	else throw new RuntimeException( "Removing unknown TestCase" );
      }

      public void testStepAdded(TestStep testStep, int index)
      {
      	TestCaseTreeNode testCaseTreeNode = (TestCaseTreeNode) getTreeModel().getTreeNode( testStep.getTestCase() );
      	testCaseTreeNode.testStepInserted( testStep, index );
      }

      public void testStepRemoved(TestStep testStep, int index)
      {
      	TestCaseTreeNode testCaseTreeNode = (TestCaseTreeNode) getTreeModel().getTreeNode( testStep.getTestCase() );
      	testCaseTreeNode.testStepRemoved( testStep, index );
      }

		public void loadTestAdded(LoadTest loadTest)
		{
      	TestCaseTreeNode testCaseTreeNode = (TestCaseTreeNode) getTreeModel().getTreeNode( loadTest.getTestCase() );
      	testCaseTreeNode.loadTestInserted( loadTest );
		}

		public void loadTestRemoved(LoadTest loadTest)
		{
      	TestCaseTreeNode testCaseTreeNode = (TestCaseTreeNode) getTreeModel().getTreeNode( loadTest.getTestCase() );
      	testCaseTreeNode.loadTestRemoved( loadTest );
		}

		public void testStepMoved(TestStep testStep, int fromIndex, int offset)
		{
      	TestCaseTreeNode testCaseTreeNode = (TestCaseTreeNode) getTreeModel().getTreeNode( testStep.getTestCase() );
      	testCaseTreeNode.testStepMoved( testStep, fromIndex, offset );
		}
   }
}
