/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support;

import java.io.PrintStream;
import java.util.ArrayList;
import java.util.List;

/**
 * Utility class for measuring and dumping timestamps
 * 
 * @author Ole.Matzura
 */

public class Timestamps
{
	private static ThreadLocal<Timestamps> timestamps = new ThreadLocal<Timestamps>();
	
	private List<String> labels = new ArrayList<String>();
	private List<Long> times = new ArrayList<Long>();

	private long startTime;
	
	private Timestamps()
	{
		startTime = System.nanoTime();
	}
	
	public static void init()
	{
		timestamps.set( new Timestamps() );
	}
	
	public static void addTimestamp( String label )
	{
		timestamps.get().add( label );
	}
	
	public synchronized static void dump()
	{
		timestamps.get().dumpResult( System.out );
	}
	
	public static void dump( PrintStream out )
	{
		timestamps.get().dumpResult( out );
	}
	
	private void add( String label )
	{
		times.add( System.nanoTime() );
		labels.add( label );
	}
	
	private void dumpResult( PrintStream out )
	{
		out.println( "Timestamps result:" );
		long last = startTime;
		for( int c = 0; c < labels.size(); c++ )
		{
			out.println( labels.get( c ) + "; " + (times.get(c)-startTime)/1000000 );
			last = times.get(c);
		}
	}

	public static long getTimeTaken()
	{
		return timestamps.get().getInternalTimeTaken();
	}

	private long getInternalTimeTaken()
	{
		if( labels.isEmpty())
			return 0;
		
		return (times.get( times.size()-1 )-startTime) / 1000000;
	}
}
