/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.action;

import java.awt.event.ActionEvent;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.Icon;

import com.eviware.soapui.support.UISupport;

public class SwingActionDelegate<T> extends AbstractAction implements PropertyChangeListener
{
	private final SoapUIAction<T> action;
	private final T target;

	public SwingActionDelegate( SoapUIAction<T> action, T target )
	{
		this( action, target, null, null );
	}
	
	public SwingActionDelegate( SoapUIAction<T> action, T target, String keyStroke )
	{
		this( action, target, null, keyStroke );
	}

	public SwingActionDelegate( SoapUIAction<T> action, T target, Icon icon )
	{
		this( action, target, icon, null );
	}
	
	public SwingActionDelegate( SoapUIAction<T> action, T target, Icon icon, String keyStroke )
	{
		super( action.getName() );
		this.action = action;
		this.target = target;
		
		if( action.getDescription() != null )
			putValue( Action.SHORT_DESCRIPTION, action.getDescription() );

		if( icon != null )
			putValue( Action.SMALL_ICON, icon );

		if( keyStroke != null )
			putValue( Action.ACCELERATOR_KEY, UISupport.getKeyStroke( keyStroke ));

		setEnabled( action.isEnabled() );
		
		action.addPropertyChangeListener( this );
	}
	
	public void actionPerformed(ActionEvent e)
	{
		action.perform( target );
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		if( evt.getPropertyName().equals( SoapUIAction.ENABLED_PROPERTY ))
			setEnabled( ((Boolean)evt.getNewValue()).booleanValue() );
	}

	public SoapUIAction<T> getAction()
	{
		return action;
	}

	public T getTarget()
	{
		return target;
	}
	
	
}
