/*
 *  soapUI, copyright (C) 2004-2007 eviware.com 
 *
 *  soapUI is free software; you can redistribute it and/or modify it under the 
 *  terms of version 2.1 of the GNU Lesser General Public License as published by 
 *  the Free Software Foundation.
 *
 *  soapUI is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without 
 *  even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 *  See the GNU Lesser General Public License for more details at gnu.org.
 */

package com.eviware.soapui.support.action;

import java.util.HashMap;
import java.util.Map;

import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.eviware.soapui.support.UISupport;

/**
 * Registry of actions.. we could probably use a spring beanfactory instead..
 * 
 * @author Ole.Matzura
 * @deprecated
 */

public class SwingActionRegistry
{
   private Map<String,ActionMapping> actions = new HashMap<String,ActionMapping>();
   private static SwingActionRegistry instance;  
   
   public SwingActionRegistry()
   {
   }
   
	public <T> Action getAction(Class<? extends SoapUIAction<T> > actionClass, T target)
	{
		ActionMapping mapping = actions.get( actionClass.getName() );
		if( mapping == null )
			return null;
		
		return new SwingActionDelegate<T>( (SoapUIAction<T>) mapping.getAction(), target, 
				mapping.getIcon(), mapping.getKeyStroke() );
	}
	
	public <T> Action getAction(Class<? extends SoapUIAction<T> > actionClass, T target, String name, String description )
	{
		Action action = getAction( actionClass, target );
		if( action != null )
		{
			if( name != null )
				action.putValue( Action.NAME, name );

			if( description != null )
				action.putValue( Action.SHORT_DESCRIPTION, description );
		}
		
		return action;
	}
	
	public void mapAction( SoapUIAction<?> action, String keyStroke, 
			String iconResource )
	{
		ActionMapping mapping = new ActionMapping( action, keyStroke, iconResource );
		actions.put( action.getClass().getName(), mapping );
	}
	
	public static SwingActionRegistry getInstance()
	{
		if( instance == null )
			instance = new SwingActionRegistry();
		
		return instance;
	}
	
	public static <T> Action get(Class<? extends SoapUIAction<T> > actionClass, T target)
	{
		return getInstance().getAction( actionClass, target );
	}
	
	public static <T> Action get(Class<? extends SoapUIAction<T> > actionClass, T target, String name, String description )
	{
		return getInstance().getAction( actionClass, target, name, description );
	}
	
	private final class ActionMapping
	{
		private final SoapUIAction<?> action;
		private final String keyStroke;
		private final String iconResource;
		private ImageIcon icon;

		public ActionMapping(SoapUIAction<?> action, String keyStroke, String iconResource)
		{
			this.action = action;
			this.keyStroke = keyStroke;
			this.iconResource = iconResource;
		}

		public SoapUIAction<?> getAction()
		{
			return action;
		}

		public String getIconResource()
		{
			return iconResource;
		}
		
		public Icon getIcon()
		{
			if( iconResource == null )
				return null;
			
			if( icon == null )
				icon = UISupport.createImageIcon( iconResource );
			
			return icon;
		}

		public String getKeyStroke()
		{
			return keyStroke;
		}
	}
}